@@ -10,18 +10,17 @@
 #include <stdlib.h>
 #include <inttypes.h>
 #include <stdatomic.h>
-#include <notcurses/direct.h>
 #include "demo.h"
 
 // (non-)ansi terminal definition-4-life
-static const int MIN_SUPPORTED_ROWS = 24;
-static const int MIN_SUPPORTED_COLS = 76; // allow a bit of margin, sigh
+static const unsigned MIN_SUPPORTED_ROWS = 24;
+static const unsigned MIN_SUPPORTED_COLS = 76; // allow a bit of margin, sigh
 
+static char *datadir;
 static int democount;
 static demoresult* results;
-static char *datadir = NOTCURSES_SHARE;
 
-static const char DEFAULT_DEMO[] = "ixetunchmdbkywgarvlsfjqzo";
+static const char DEFAULT_DEMO[] = "ixetunchdmbkywjgarvlsfqzo";
 
 atomic_bool interrupted = ATOMIC_VAR_INIT(false);
 // checked following demos, whether aborted, failed, or otherwise
@@ -44,11 +43,7 @@ const demoresult* demoresult_lookup(int idx){
 }
 
 char* find_data(const char* datum){
-  char* path = malloc(strlen(datadir) + 1 + strlen(datum) + 1);
-  strcpy(path, datadir);
-  strcat(path, "/");
-  strcat(path, datum);
-  return path;
+  return notcurses_data_path(datadir, datum);
 }
 
 float delaymultiplier = 1;
@@ -62,8 +57,9 @@ struct timespec demodelay = {
 // the "jungle" demo has non-free material embedded into it, and is thus
 // entirely absent (can't just be disabled). supply a stub here.
 #ifdef DFSG_BUILD
-int jungle_demo(struct notcurses* nc){
+int jungle_demo(struct notcurses* nc, uint64_t startns){
   (void)nc;
+  (void)startns;
   return -1;
 }
 #endif
@@ -72,7 +68,7 @@ int jungle_demo(struct notcurses* nc){
 // be disabled) or a non-multimedia build (all images/videos must be disabled).
 static struct {
   const char* name;
-  int (*fxn)(struct notcurses*);
+  int (*fxn)(struct notcurses*, uint64_t startns);
   bool dfsg_disabled;             // disabled for DFSG builds
 } demos[26] = {
   { "animate", animate_demo, false, },
@@ -82,110 +78,117 @@ static struct {
   { "eagle", eagle_demo, true, },
   { "fission", fission_demo, false, },
   { "grid", grid_demo, false, },
-  { "highcon", highcontrast_demo, false, },
-  { "intro", intro, false, },
+  { "highcon", highcon_demo, false, },
+  { "intro", intro_demo, false, },
   { "jungle", jungle_demo, true, },
   { "keller", keller_demo, true, },
   { "luigi", luigi_demo, true, },
   { "mojibake", mojibake_demo, false, },
   { "normal", normal_demo, false, },
-  { "outro", outro, false, },
-  { NULL, NULL, false, }, // pango
+  { "outro", outro_demo, false, },
+  { NULL, NULL, false, }, // it's a secret to everyone
   { "qrcode", qrcode_demo, false, }, // is blank without USE_QRCODEGEN
   { "reel", reel_demo, false, },
-  { "sliders", sliding_puzzle_demo, false, },
+  { "sliders", sliders_demo, false, },
   { "trans", trans_demo, false, },
-  { "uniblock", unicodeblocks_demo, false, },
+  { "uniblock", uniblock_demo, false, },
   { "view", view_demo, true, },
-  { "whiteout", witherworm_demo, false, },
+  { "whiteout", whiteout_demo, false, },
   { "xray", xray_demo, false, },
   { "yield", yield_demo, false, },
   { "zoo", zoo_demo, false, },
 };
 
 static void
-usage_option(FILE* out, struct ncdirect* n, const char* op){
-  if(n) ncdirect_set_fg_rgb8(n, 0x80, 0x80, 0x80);
-  fprintf(out, " [ ");
-  if(n) ncdirect_set_fg_rgb8(n, 0xff, 0xff, 0x80);
-  fprintf(out, "%s", op);
-  if(n) ncdirect_set_fg_rgb8(n, 0x80, 0x80, 0x80);
-  fprintf(out, " ] ");
-  if(n) ncdirect_set_fg_rgb8(n, 0xff, 0xff, 0xff);
+usage_option(struct ncplane* n, const char* op){
+  ncplane_set_fg_rgb8(n, 0x80, 0x80, 0x80);
+  ncplane_printf(n, " [ ");
+  ncplane_set_fg_rgb8(n, 0xff, 0xff, 0x80);
+  ncplane_printf(n, "%s", op);
+  ncplane_set_fg_rgb8(n, 0x80, 0x80, 0x80);
+  ncplane_printf(n, " ] ");
+  ncplane_set_fg_rgb8(n, 0xff, 0xff, 0xff);
 }
 
 static void
-usage_expo(FILE* out, struct ncdirect* n, const char* op, const char* expo){
-  if(n) ncdirect_set_fg_rgb8(n, 0xff, 0xff, 0x80);
-  fprintf(out, " %s: ", op);
-  if(n) ncdirect_set_fg_rgb8(n, 0xff, 0xff, 0xff);
-  fprintf(out, "%s\n", expo);
+usage_expo(struct ncplane* n, const char* op, const char* expo){
+  ncplane_set_fg_rgb8(n, 0xff, 0xff, 0x80);
+  ncplane_printf(n, " %s: ", op);
+  ncplane_set_fg_rgb8(n, 0xff, 0xff, 0xff);
+  ncplane_printf(n, "%s\n", expo);
 }
 
-// FIXME stylize this a little
 static void
 usage(const char* exe, int status){
   FILE* out = status == EXIT_SUCCESS ? stdout : stderr;
-  struct ncdirect* n = ncdirect_init(NULL, out, 0);
-  if(n) ncdirect_set_fg_rgb8(n, 0x00, 0xc0, 0xc0);
-  fprintf(out, "usage: ");
-  if(n) ncdirect_set_fg_rgb8(n, 0x80, 0xff, 0x80);
-  fprintf(out, "%s ", exe);
+  struct notcurses_options opts = {0};
+  opts.flags = NCOPTION_CLI_MODE
+               | NCOPTION_DRAIN_INPUT
+               | NCOPTION_SUPPRESS_BANNERS;
+  struct notcurses* nc = notcurses_init(&opts, out);
+  if(!nc){
+    exit(status);
+  }
+  struct ncplane* n = notcurses_stdplane(nc);
+  ncplane_set_fg_rgb8(n, 0x00, 0xc0, 0xc0);
+  ncplane_putstr(n, "usage: ");
+  ncplane_set_fg_rgb8(n, 0x80, 0xff, 0x80);
+  ncplane_printf(n, "%s ", exe);
   const char* options[] = { "-hVkc", "-m margins", "-p path", "-l loglevel",
-                            "-d mult", "-J jsonfile", "-f renderfile", "demospec",
+                            "-d mult", "-J jsonfile", "demospec",
                             NULL };
   for(const char** op = options ; *op ; ++op){
-    usage_option(out, n, *op);
+    usage_option(n, *op);
   }
-  fprintf(out, "\n\n");
-  if(n) ncdirect_set_fg_rgb8(n, 0xff, 0xff, 0xff);
+  ncplane_putstr(n, "\n\n");
+  ncplane_set_fg_rgb8(n, 0xff, 0xff, 0xff);
   const char* optexpo[] = {
     "-h|--help", "this message",
     "-V|--version", "print program name and version",
     "-k", "keep screen; do not switch to alternate",
     "-d", "delay multiplier (non-negative float)",
     "-J", "emit JSON summary to file",
-    "-f", "render to file (in addition to stdout)",
     "-c", "constant PRNG seed, useful for benchmarking",
     "-m", "margin, or 4 comma-separated margins",
     NULL
   };
   for(const char** op = optexpo ; *op ; op += 2){
     const char* expo = op[1];
-    usage_expo(out, n, *op, expo);
+    usage_expo(n, *op, expo);
   }
-  if(n) ncdirect_set_fg_rgb8(n, 0xff, 0xff, 0x80);
-  fprintf(out, " -l:");
-  if(n) ncdirect_set_fg_rgb8(n, 0xff, 0xff, 0xff);
-  fprintf(out, " logging level (%d: silent..%d: manic)\n", NCLOGLEVEL_SILENT, NCLOGLEVEL_TRACE);
-  if(n) ncdirect_set_fg_rgb8(n, 0xff, 0xff, 0x80);
-  fprintf(out, " -p:");
-  if(n) ncdirect_set_fg_rgb8(n, 0xff, 0xff, 0xff);
-  fprintf(out, " data file path (default: %s)\n", NOTCURSES_SHARE);
-  fprintf(out, "\nspecify demos via their first letter. repetitions are allowed.\n");
-  if(n) ncdirect_set_fg_rgb8(n, 0x80, 0xff, 0x80);
-  fprintf(out, " default spec: %s\n\n", DEFAULT_DEMO);
-  if(n) ncdirect_set_fg_rgb8(n, 0xff, 0xff, 0xff);
+  ncplane_set_fg_rgb8(n, 0xff, 0xff, 0x80);
+  ncplane_printf(n, " -l:");
+  ncplane_set_fg_rgb8(n, 0xff, 0xff, 0xff);
+  ncplane_printf(n, " logging level (%d: silent..%d: manic)\n", NCLOGLEVEL_SILENT, NCLOGLEVEL_TRACE);
+  ncplane_set_fg_rgb8(n, 0xff, 0xff, 0x80);
+  ncplane_printf(n, " -p:");
+  ncplane_set_fg_rgb8(n, 0xff, 0xff, 0xff);
+  ncplane_printf(n, " data file path (default: %s)\n", notcurses_data_dir());
+  ncplane_printf(n, "\nspecify demos via their first letter. repetitions are allowed.\n");
+  ncplane_set_fg_rgb8(n, 0x80, 0xff, 0x80);
+  ncplane_printf(n, " default spec: %s\n\n", DEFAULT_DEMO);
+  ncplane_set_fg_rgb8(n, 0xff, 0xff, 0xff);
   int printed = 0;
   for(size_t i = 0 ; i < sizeof(demos) / sizeof(*demos) ; ++i){
     if(demos[i].name){
       if(printed % 5 == 0){
-        fprintf(out, " ");
+        ncplane_printf(n, " ");
       }
       // U+24D0: CIRCLED LATIN SMALL LETTER A
-      if(n) ncdirect_set_fg_rgb8(n, 0xff, 0xff, 0x80);
-      fprintf(out, "%lc ", *demos[i].name - 'a' + 0x24d0);
-      if(n) ncdirect_set_fg_rgb8(n, 0xff, 0xff, 0xff);
-      fprintf(out, "%-*.*s", 8, 8, demos[i].name + 1);
+      ncplane_set_fg_rgb8(n, 0xff, 0xff, 0x80);
+      ncplane_printf(n, "%lc ", (wint_t)(*demos[i].name - 'a' + 0x24d0));
+      ncplane_set_fg_rgb8(n, 0xff, 0xff, 0xff);
+      ncplane_printf(n, "%-*.*s", 8, 8, demos[i].name + 1);
       if(++printed % 5 == 0){
-        fprintf(out, "\n");
+        ncplane_printf(n, "\n");
       }
     }
   }
   if(printed % 5){
-    fprintf(out, "\n");
+    ncplane_printf(n, "\n");
   }
-  ncdirect_stop(n);
+  notcurses_render(nc);
+  notcurses_stop(nc);
   exit(status);
 }
 
@@ -198,9 +201,7 @@ ext_demos(struct notcurses* nc, const char* spec){
   }
   memset(results, 0, sizeof(*results) * strlen(spec));
   democount = strlen(spec);
-  struct timespec start, now;
-  clock_gettime(CLOCK_MONOTONIC, &start);
-  uint64_t prevns = timespec_to_ns(&start);
+  uint64_t prevns = clock_getns(CLOCK_MONOTONIC);
   for(size_t i = 0 ; i < strlen(spec) ; ++i){
     results[i].selector = spec[i];
   }
@@ -221,11 +222,10 @@ ext_demos(struct notcurses* nc, const char* spec){
     uint64_t stdc = NCCHANNELS_INITIALIZER(0, 0, 0, 0, 0, 0);
     ncplane_set_base(n, "", 0, stdc);
 
-    hud_schedule(demos[idx].name);
-    ret = demos[idx].fxn(nc);
+    hud_schedule(demos[idx].name, prevns);
+    ret = demos[idx].fxn(nc, prevns);
     notcurses_stats_reset(nc, &results[i].stats);
-    clock_gettime(CLOCK_MONOTONIC, &now);
-    uint64_t nowns = timespec_to_ns(&now);
+    uint64_t nowns = clock_getns(CLOCK_MONOTONIC);
     results[i].timens = nowns - prevns;
     prevns = nowns;
     results[i].result = ret;
@@ -243,7 +243,6 @@ ext_demos(struct notcurses* nc, const char* spec){
 static const char*
 handle_opts(int argc, char** argv, notcurses_options* opts, FILE** json_output){
   bool constant_seed = false;
-  char *renderfile = NULL;
   *json_output = NULL;
   int c;
   const struct option longopts[] = {
@@ -252,7 +251,7 @@ handle_opts(int argc, char** argv, notcurses_options* opts, FILE** json_output){
     { .name = NULL, .has_arg = 0, .flag = NULL, .val = 0, },
   };
   int lidx;
-  while((c = getopt_long(argc, argv, "VhckJ:l:r:d:f:p:m:", longopts, &lidx)) != EOF){
+  while((c = getopt_long(argc, argv, "VhckJ:l:d:p:m:", longopts, &lidx)) != EOF){
     switch(c){
       case 'h':
         usage(*argv, EXIT_SUCCESS);
@@ -297,20 +296,8 @@ handle_opts(int argc, char** argv, notcurses_options* opts, FILE** json_output){
       case 'k':
         opts->flags |= NCOPTION_NO_ALTERNATE_SCREEN;
         break;
-      case 'f':
-        if(opts->renderfp){
-          fprintf(stderr, "-f may only be supplied once\n");
-          usage(*argv, EXIT_FAILURE);
-        }
-        if((opts->renderfp = fopen(optarg, "wb")) == NULL){
-          usage(*argv, EXIT_FAILURE);
-        }
-        break;
       case 'p':
-        datadir = optarg;
-        break;
-      case 'r':
-        renderfile = optarg;
+        datadir = strdup(optarg);
         break;
       case 'd':{
         float f;
@@ -334,45 +321,53 @@ handle_opts(int argc, char** argv, notcurses_options* opts, FILE** json_output){
   if(!constant_seed){
     srand(time(NULL)); // a classic blunder lol
   }
-  if(renderfile){
-    opts->renderfp = fopen(renderfile, "wb");
-    if(opts->renderfp == NULL){
-      fprintf(stderr, "Error opening %s for write\n", renderfile);
-      usage(*argv, EXIT_FAILURE);
-    }
-  }
   if(optind < argc - 1){
     fprintf(stderr, "Extra argument: %s\n", argv[optind + 1]);
     usage(*argv, EXIT_FAILURE);
   }
+  if(datadir == NULL){
+    datadir = notcurses_data_dir();
+  }
   const char* spec = argv[optind];
   return spec;
 }
 
 static int
-table_segment_color(struct ncdirect* nc, const char* str, const char* delim, unsigned color){
-  ncdirect_set_fg_rgb(nc, color);
-  fputs(str, stdout);
-  ncdirect_set_fg_rgb8(nc, 178, 102, 255);
-  fputs(delim, stdout);
+table_segment_color(struct ncplane* n, const char* str, const char* delim,
+                    const char* ascdelim, unsigned color){
+  ncplane_set_fg_rgb(n, color);
+  if(ncplane_putstr(n, str) < 0){
+    return -1;
+  }
+  ncplane_set_fg_rgb8(n, 178, 102, 255);
+  if(notcurses_canutf8(ncplane_notcurses(n))){
+    if(ncplane_putstr(n, delim) < 0){
+      return -1;
+    }
+  }else{
+    if(ncplane_putstr(n, ascdelim) < 0){
+      return -1;
+    }
+  }
   return 0;
 }
 
 static int
-table_segment(struct ncdirect* nc, const char* str, const char* delim){
-  return table_segment_color(nc, str, delim, 0xffffff);
+table_segment(struct ncplane* n, const char* str, const char* delim,
+              const char* ascdelim){
+  return table_segment_color(n, str, delim, ascdelim, 0xffffff);
 }
 
 static int
-table_printf(struct ncdirect* nc, const char* delim, const char* fmt, ...){
-  ncdirect_set_fg_rgb8(nc, 0xD4, 0xAF, 0x37);
+table_printf(struct ncplane* n, const char* delim, const char* fmt, ...){
+  ncplane_set_fg_rgb8(n, 0xD4, 0xAF, 0x37);
   va_list va;
   va_start(va, fmt);
-  vfprintf(stdout, fmt, va);
+  int r = ncplane_vprintf(n, fmt, va);
   va_end(va);
-  ncdirect_set_fg_rgb8(nc, 178, 102, 255);
-  fputs(delim, stdout);
-  return 0;
+  ncplane_set_fg_rgb8(n, 178, 102, 255);
+  ncplane_putstr(n, delim);
+  return r;
 }
 
 static int
@@ -385,7 +380,7 @@ summary_json(FILE* f, const char* spec, int rows, int cols){
       continue;
     }
     ret |= (fprintf(f, "\"%s\":{\"bytes\":\"%"PRIu64"\",\"frames\":\"%"PRIu64"\",\"ns\":\"%"PRIu64"\"}%s",
-                    demos[results[i].selector - 'a'].name, results[i].stats.render_bytes,
+                    demos[results[i].selector - 'a'].name, results[i].stats.raster_bytes,
                     results[i].stats.renders, results[i].timens, i < strlen(spec) - 1 ? "," : "") < 0);
   }
   ret |= (fprintf(f, "}}}\n") < 0);
@@ -393,35 +388,47 @@ summary_json(FILE* f, const char* spec, int rows, int cols){
 }
 
 static int
-summary_table(struct ncdirect* nc, const char* spec, bool canimage, bool canvideo){
+summary_table(struct notcurses* nc, const char* spec, bool canimage, bool canvideo){
+  notcurses_leave_alternate_screen(nc);
+  struct ncplane* n = notcurses_stdplane(nc);
+  ncplane_set_bg_default(n);
+  ncplane_set_scrolling(n, true);
   bool failed = false;
   uint64_t totalbytes = 0;
   long unsigned totalframes = 0;
-  uint64_t totalrenderns = 0;
-  uint64_t totalwriteoutns = 0;
-  printf("\n");
-  table_segment(nc, "             runtime", "│");
-  table_segment(nc, " frames", "│");
-  table_segment(nc, "output(B)", "│");
-  table_segment(nc, "    FPS", "│");
-  table_segment(nc, "%r", "│");
-  table_segment(nc, "%a", "│");
-  table_segment(nc, "%w", "│");
-  table_segment(nc, "TheoFPS", "║\n══╤════════╤════════╪═══════╪═════════╪═══════╪══╪══╪══╪═══════╣\n");
-  char timebuf[PREFIXSTRLEN + 1];
-  char tfpsbuf[PREFIXSTRLEN + 1];
-  char totalbuf[BPREFIXSTRLEN + 1];
+  ncplane_putchar(n, '\n');
+  // FIXME this shouldn't be necessary, but without it, late in 2.4.x we
+  // stopped printing the table header. see #2389.
+  notcurses_render(nc);
+  const char* sep;
+  if(notcurses_canutf8(ncplane_notcurses(n))){
+    sep = u8"│";
+  }else{
+    sep = "|";
+  }
+  table_segment(n, "             runtime", "│", "|");
+  table_segment(n, " frames", "│", "|");
+  table_segment(n, "output(B)", "│", "|");
+  table_segment(n, "    FPS", "│", "|");
+  table_segment(n, "%r", "│", "|");
+  table_segment(n, "%a", "│", "|");
+  table_segment(n, "%w", "│", "|");
+  table_segment(n, "TheoFPS", "║\n══╤════════╤════════╪═══════╪═════════╪═══════╪══╪══╪══╪═══════╣\n",
+                "|\n--+--------+--------+-------+---------+-------+--+--+--+-------|\n");
+  char timebuf[NCPREFIXSTRLEN + 1];
+  char tfpsbuf[NCPREFIXSTRLEN + 1];
+  char totalbuf[NCBPREFIXSTRLEN + 1];
   uint64_t nsdelta = 0;
   for(size_t i = 0 ; i < strlen(spec) ; ++i){
     nsdelta += results[i].timens;
-    qprefix(results[i].timens, NANOSECS_IN_SEC, timebuf, 0);
-    bprefix(results[i].stats.render_bytes, 1, totalbuf, 0);
+    ncqprefix(results[i].timens, NANOSECS_IN_SEC, timebuf, 0);
+    ncbprefix(results[i].stats.raster_bytes, 1, totalbuf, 0);
     uint64_t divisor = results[i].stats.render_ns + results[i].stats.writeout_ns + results[i].stats.raster_ns;
     if(divisor){
-      qprefix((uintmax_t)results[i].stats.writeouts * NANOSECS_IN_SEC * 1000 / divisor,
+      ncqprefix((uintmax_t)results[i].stats.writeouts * NANOSECS_IN_SEC * 1000 / divisor,
               1000, tfpsbuf, 0);
     }else{
-      qprefix(0, NANOSECS_IN_SEC, tfpsbuf, 0);
+      ncqprefix(0, NANOSECS_IN_SEC, tfpsbuf, 0);
     }
     uint32_t rescolor;
     if(results[i].result < 0){
@@ -433,60 +440,59 @@ summary_table(struct ncdirect* nc, const char* spec, bool canimage, bool canvide
     }else{
       rescolor = 0x32CD32;
     }
-    ncdirect_set_fg_rgb(nc, rescolor);
-    printf("%2zu", i + 1);
-    ncdirect_set_fg_rgb8(nc, 178, 102, 255);
-    printf("│");
-    ncdirect_set_fg_rgb(nc, rescolor);
-    printf("%8s", demos[results[i].selector - 'a'].name);
-    ncdirect_set_fg_rgb8(nc, 178, 102, 255);
-    printf("│%*ss│%7ju│%*s│%7.1f│%2jd│%2jd│%2jd│%*s║",
-           PREFIXFMT(timebuf), (uintmax_t)(results[i].stats.renders),
-           BPREFIXFMT(totalbuf),
+    ncplane_set_fg_rgb(n, rescolor);
+    ncplane_printf(n, "%2llu", (unsigned long long)(i + 1)); // windows has %zu problems
+    ncplane_set_fg_rgb8(n, 178, 102, 255);
+    ncplane_putegc(n, sep, NULL);
+    ncplane_set_fg_rgb(n, rescolor);
+    ncplane_printf(n, "%8s", demos[results[i].selector - 'a'].name);
+    ncplane_set_fg_rgb8(n, 178, 102, 255);
+    ncplane_printf(n, "%s%*ss%s%7" PRIu64 "%s%*s%s%7.1f%s%2" PRId64 "%s%2" PRId64 "%s%2" PRId64 "%s%*s%s",
+           sep, NCPREFIXFMT(timebuf), sep,
+           results[i].stats.renders, sep,
+           NCBPREFIXFMT(totalbuf), sep,
            results[i].timens ?
-            results[i].stats.renders / ((double)results[i].timens / NANOSECS_IN_SEC) : 0.0,
-           (uintmax_t)(results[i].timens ?
-            results[i].stats.render_ns * 100 / results[i].timens : 0),
-           (uintmax_t)(results[i].timens ?
-            results[i].stats.raster_ns * 100 / results[i].timens : 0),
-           (uintmax_t)(results[i].timens ?
-            results[i].stats.writeout_ns * 100 / results[i].timens : 0),
-           PREFIXFMT(tfpsbuf));
-    ncdirect_set_fg_rgb(nc, rescolor);
-    printf("%s\n", results[i].result < 0 ? "FAILED" :
-            results[i].result > 0 ? "ABORTED" :
-             !results[i].stats.renders ? "SKIPPED"  : "");
+            results[i].stats.renders / ((double)results[i].timens / NANOSECS_IN_SEC) : 0.0, sep,
+           (results[i].timens ?
+            results[i].stats.render_ns * 100 / results[i].timens : 0), sep,
+           (results[i].timens ?
+            results[i].stats.raster_ns * 100 / results[i].timens : 0), sep,
+           (results[i].timens ?
+            results[i].stats.writeout_ns * 100 / results[i].timens : 0), sep,
+           NCPREFIXFMT(tfpsbuf),
+           notcurses_canutf8(ncplane_notcurses(n)) ? "║" : "|");
+    ncplane_set_fg_rgb(n, rescolor);
+    ncplane_printf(n, "%s\n", results[i].result < 0 ? "FAILED" :
+                   results[i].result > 0 ? "ABORTED" :
+                   !results[i].stats.renders ? "SKIPPED"  : "");
     if(results[i].result < 0){
       failed = true;
     }
     totalframes += results[i].stats.renders;
-    totalbytes += results[i].stats.render_bytes;
-    totalrenderns += results[i].stats.render_ns;
-    totalwriteoutns += results[i].stats.writeout_ns;
+    totalbytes += results[i].stats.raster_bytes;
   }
-  qprefix(nsdelta, NANOSECS_IN_SEC, timebuf, 0);
-  bprefix(totalbytes, 1, totalbuf, 0);
-  table_segment(nc, "", "══╧════════╧════════╪═══════╪═════════╪═══════╧══╧══╧══╧═══════╝\n");
-  printf("            ");
-  table_printf(nc, "│", "%*ss", PREFIXFMT(timebuf));
-  table_printf(nc, "│", "%7lu", totalframes);
-  table_printf(nc, "│", "%*s", BPREFIXFMT(totalbuf));
-  //table_printf(nc, "│", "%7.1f", nsdelta ? totalframes / ((double)nsdelta / NANOSECS_IN_SEC) : 0);
-  printf("\n");
-  ncdirect_set_fg_rgb8(nc, 0xff, 0xb0, 0xb0);
-  fflush(stdout); // in case we print to stderr below, we want color from above
-  if(failed){
-    fprintf(stderr, "\nError running demo.\nIs \"%s\" the correct data path? Supply it with -p.\n", datadir);
-  }
-  ncdirect_set_fg_rgb8(nc, 0xfe, 0x20, 0x76); // PANTONE Strong Red C + 3x0x20
-  fflush(stdout); // in case we print to stderr below, we want color from above
+  ncqprefix(nsdelta, NANOSECS_IN_SEC, timebuf, 0);
+  ncbprefix(totalbytes, 1, totalbuf, 0);
+  table_segment(n, "", "══╧════════╧════════╪═══════╪═════════╪═══════╧══╧══╧══╧═══════╝\n",
+                "--+--------+--------+-------+---------+-------+--+--+--+-------+\n");
+  ncplane_putstr(n, "            ");
+  table_printf(n, sep, "%*ss", NCPREFIXFMT(timebuf));
+  table_printf(n, sep, "%7lu", totalframes);
+  table_printf(n, sep, "%*s", NCBPREFIXFMT(totalbuf));
+  //table_printf(nc, sep, "%7.1f", nsdelta ? totalframes / ((double)nsdelta / NANOSECS_IN_SEC) : 0);
+  ncplane_putchar(n, '\n');
+  ncplane_set_fg_rgb8(n, 0xfe, 0x20, 0x76); // PANTONE Strong Red C + 3x0x20
 #ifdef DFSG_BUILD
-  fprintf(stderr, "\nDFSG version. Some demos are unavailable.\n");
+  ncplane_putstr(n, "\nDFSG version. Some demos are unavailable.\n");
 #endif
   if(!canimage){
-    fprintf(stderr, "\nNo multimedia support. Some demos are unavailable.\n");
+    ncplane_putstr(n, "\nNo multimedia support. Some demos are unavailable.\n");
   }else if(!canvideo){
-    fprintf(stderr, "\nNo video support. Some demos are unavailable.\n");
+    ncplane_putstr(n, "\nNo video support. Some demos are unavailable.\n");
+  }
+  ncplane_set_fg_rgb8(n, 0xff, 0xb0, 0xb0);
+  if(failed){
+    ncplane_printf(n, "\nError running demo.\nIs \"%s\" the correct data path? Supply it with -p.\n", datadir);
   }
   return failed;
 }
@@ -505,14 +511,16 @@ scrub_stdplane(struct notcurses* nc){
 }
 
 int main(int argc, char** argv){
+#ifndef __MINGW32__
   sigset_t sigmask;
   // ensure SIGWINCH is delivered only to a thread doing input
   sigemptyset(&sigmask);
   sigaddset(&sigmask, SIGWINCH);
   pthread_sigmask(SIG_BLOCK, &sigmask, NULL);
+#endif
   const char* spec;
   FILE* json = NULL; // emit JSON summary to this file? (-J)
-  notcurses_options nopts = {};
+  notcurses_options nopts = {0};
   if((spec = handle_opts(argc, argv, &nopts, &json)) == NULL){
     if(argv[optind] != NULL){
       usage(*argv, EXIT_FAILURE);
@@ -533,10 +541,10 @@ int main(int argc, char** argv){
   if((nc = notcurses_init(&nopts, NULL)) == NULL){
     return EXIT_FAILURE;
   }
-  notcurses_mouse_enable(nc);
+  notcurses_mice_enable(nc, NCMICE_BUTTON_EVENT | NCMICE_DRAG_EVENT);
   const bool canimage = notcurses_canopen_images(nc);
   const bool canvideo = notcurses_canopen_videos(nc);
-  int dimx, dimy;
+  unsigned dimx, dimy;
   ncplane_dim_yx(notcurses_stdplane(nc), &dimy, &dimx);
   if(input_dispatcher(nc)){
     goto err;
@@ -585,37 +593,25 @@ int main(int argc, char** argv){
   }while(restart_demos);
   ncmenu_destroy(menu);
   stop_input();
+  notcurses_render(nc); // rid ourselves of any remaining demo output
+  int r = summary_table(nc, spec, canimage, canvideo);
+  notcurses_render(nc); // render our summary table
+  free(results);
   if(notcurses_stop(nc)){
     return EXIT_FAILURE;
   }
-  if(nopts.renderfp){
-    if(fclose(nopts.renderfp)){
-      fprintf(stderr, "Warning: error closing renderfile\n");
-    }
-  }
-  struct ncdirect* ncd = ncdirect_init(NULL, stdout, 0);
-  if(!ncd){
+  if(json && summary_json(json, spec, dimy, dimx)){
     return EXIT_FAILURE;
   }
-  if(json && summary_json(json, spec, ncdirect_dim_y(ncd), ncdirect_dim_x(ncd))){
-    return EXIT_FAILURE;
-  }
-  // reinitialize without alternate screen to do some coloring
-  if(summary_table(ncd, spec, canimage, canvideo)){
-    ncdirect_stop(ncd);
-    return EXIT_FAILURE;
-  }
-  free(results);
-  ncdirect_stop(ncd);
-  return EXIT_SUCCESS;
+  free(datadir);
+  return r ? EXIT_FAILURE : EXIT_SUCCESS;
 
 err:
   stop_input();
-  notcurses_term_dim_yx(nc, &dimy, &dimx);
   notcurses_stop(nc);
   if(dimy < MIN_SUPPORTED_ROWS || dimx < MIN_SUPPORTED_COLS){
     fprintf(stderr, "At least a %dx%d terminal is required (current: %dx%d)\n",
-            MIN_SUPPORTED_COLS, MIN_SUPPORTED_ROWS, dimx, dimy);
+            MIN_SUPPORTED_ROWS, MIN_SUPPORTED_COLS, dimy, dimx);
   }
   return EXIT_FAILURE;
 }
