@@ -14,7 +14,7 @@
 static struct ncplane*
 mathplane(struct notcurses* nc){
   struct ncplane* stdn = notcurses_stdplane(nc);
-  int dimx, dimy;
+  unsigned dimx, dimy;
   notcurses_term_dim_yx(nc, &dimy, &dimx);
   const int HEIGHT = 9;
   const int WIDTH = dimx;
@@ -184,7 +184,7 @@ message(struct ncplane* n, int maxy, int maxx, int num, int total,
   ncplane_putegc_yx(n, 4, 17, "┬", NULL);
   ncplane_putegc_yx(n, 5, 17, "│", NULL);
   ncplane_putegc_yx(n, 6, 17, "╰", NULL);
-  nccell hl = CELL_TRIVIAL_INITIALIZER;
+  nccell hl = NCCELL_TRIVIAL_INITIALIZER;
   nccell_load(n, &hl, "─");
   nccell_set_fg_rgb8(&hl, 255, 255, 255);
   nccell_set_bg_rgb8(&hl, 32, 64, 32);
@@ -210,13 +210,58 @@ message(struct ncplane* n, int maxy, int maxx, int num, int total,
   ncplane_printf_yx(n, 1, 4, " %03dx%03d (%d/%d) ", maxx, maxy, num + 1, total);
   ncplane_off_styles(n, NCSTYLE_ITALIC);
   ncplane_set_fg_rgb8(n, 224, 128, 224);
-  ncplane_putstr_yx(n, 3, 1, " 🎆🔥 unicode 13, resize awareness, 24b truecolor…🔥🎆 ");
+  ncplane_putstr_yx(n, 3, 1, " 🎆🔥 unicode 14, resize awareness, 24b truecolor…🔥🎆 ");
   ncplane_set_fg_rgb8(n, 255, 255, 255);
   return 0;
 }
 
+// print the EGCs of the UTF8 string on the standard plane, wrapping around
+// the borders. the plane is not set to scroll, so we handle it ourselves.
+static int
+dostring(struct ncplane* n, const char** s, uint32_t rgb,
+         unsigned maxy, unsigned maxx,
+         unsigned* egcs, unsigned* cols, unsigned* bytes){
+  size_t idx = 0;
+  unsigned y, x;
+  ncplane_cursor_yx(n, &y, &x);
+  while((*s)[idx]){ // each multibyte char of string
+    if(ncplane_set_fg_rgb8(n, ncchannel_r(rgb), ncchannel_g(rgb), ncchannel_b(rgb))){
+      return -1;
+    }
+    if(x >= maxx){
+      x = 0;
+      ++y;
+    }
+    if(y >= maxy){
+      break;
+    }
+    wchar_t wcs;
+    mbstate_t mbstate;
+    memset(&mbstate, 0, sizeof(mbstate));
+    int eaten = mbrtowc(&wcs, &(*s)[idx], MB_CUR_MAX + 1, &mbstate);
+    if(eaten < 0){
+      return -1;
+    }
+    if(iswspace(wcs)){
+      idx += eaten;
+      continue;
+    }
+    size_t ulen = 0;
+    int r = 0;
+    if((r = ncplane_putegc(n, &(*s)[idx], &ulen)) <= 0){
+      return 0;
+    }
+    ncplane_cursor_yx(n, &y, &x);
+    idx += ulen;
+    *bytes += ulen;
+    *cols += r;
+    ++*egcs;
+  }
+  return 0;
+}
+
 // Much of this text comes from http://kermitproject.org/utf8.html
-int witherworm_demo(struct notcurses* nc){
+int whiteout_demo(struct notcurses* nc, uint64_t startns){
   static const char* strs[] = {
     "Война и мир",
     "Бра́тья Карама́зовы",
@@ -355,6 +400,7 @@ int witherworm_demo(struct notcurses* nc){
     "Jeg kan spise glas, det gør ikke ondt på mig",
     "㎚㎛㎜㎝㎞㎟㎠㎡㎢㎣㎤㎥㎦㎕㎖㎗㎘㏄㎰㎱㎲㎳㎍㎎㎏㎅㎆㏔㎇㎐㎑㎒㎓㎔㎮㎯",
     "Æ ka æe glass uhen at det go mæ naue",
+    // FIXME this one
     "က္ယ္ဝန္တော္၊က္ယ္ဝန္မ မ္ယက္စားနုိင္သည္။ ၎က္ရောင္ ထိခုိက္မ္ဟု မရ္ဟိပာ။",
     "ကျွန်တော် ကျွန်မ မှန်စားနိုင်တယ်။ ၎င်းကြောင့် ထိခိုက်မှုမရှိပါ။ ",
     "Tôi có thể ăn thủy tinh mà không hại gì",
@@ -458,6 +504,7 @@ int witherworm_demo(struct notcurses* nc){
   bool initial_scroll = ncplane_scrolling_p(n);
   ncplane_set_scrolling(n, true);
   ncplane_erase(n);
+  (void)startns; // FIXME integrate
   for(i = 0 ; i < screens ; ++i){
     uint32_t key = NCKEY_INVALID;
     nccell c;
@@ -468,67 +515,26 @@ int witherworm_demo(struct notcurses* nc){
       const int start = starts[i];
       int step = steps[i];
       nccell_init(&c);
-      int y, x, maxy, maxx;
+      unsigned maxy, maxx;
       ncplane_dim_yx(n, &maxy, &maxx); // might resize
-      int rgb = start;
-      int bytes_out = 0;
-      int egcs_out = 0;
-      int cols_out = 0;
-      y = 1;
-      x = 0;
+      uint32_t rgb = start;
+      unsigned bytes_out = 0;
+      unsigned egcs_out = 0;
+      unsigned cols_out = 0;
+      unsigned y = 1;
+      unsigned x = 0;
       if(ncplane_cursor_move_yx(n, y, x)){
         return -1;
       }
       ncplane_set_bg_rgb8(n, 20, 20, 20);
       do{ // we fill up the screen, however large, bouncing around our strtable
         s = strs + rand() % ((sizeof(strs) / sizeof(*strs)) - 1);
-        size_t idx = 0;
-        ncplane_cursor_yx(n, &y, &x);
-        while((*s)[idx]){ // each multibyte char of string
-          if(ncplane_set_fg_rgb8(n, ncchannel_r(rgb), ncchannel_g(rgb), ncchannel_b(rgb))){
-            return -1;
-          }
-          if(x >= maxx){
-            x = 0;
-            ++y;
-          }
-          if(y >= maxy){
-            break;
-          }
-          wchar_t wcs;
-          mbstate_t mbstate;
-          memset(&mbstate, 0, sizeof(mbstate));
-          int eaten = mbrtowc(&wcs, &(*s)[idx], MB_CUR_MAX + 1, &mbstate);
-          if(eaten < 0){
-            return -1;
-          }
-          if(iswspace(wcs)){
-            idx += eaten;
-            continue;
-          }
-          int ulen = 0;
-          int r;
-          if(wcwidth(wcs) <= maxx - x){
-            if((r = ncplane_putegc(n, &(*s)[idx], &ulen)) <= 0){
-              if(ulen < 0){
-                return -1;
-              }else if(ulen == 0){
-                break; // FIXME work around missing unicode
-              }
-            }
-          }else{
-            if((r = ncplane_putchar(n, '#')) < 1){
-              return -1;
-            }
-          }
-          ncplane_cursor_yx(n, &y, &x);
-          idx += ulen;
-          bytes_out += ulen;
-          cols_out += r;
-          ++egcs_out;
+        if(dostring(n, s, rgb, maxy, maxx, &egcs_out, &cols_out, &bytes_out)){
+          return -1;
         }
         rgb += step;
-      }while(y < maxy);
+        ncplane_cursor_yx(n, &y, &x);
+      }while(y < (maxy - 1) || x < (maxx - 2));
       struct ncplane* math = mathplane(nc);
       if(math == NULL){
         return -1;
@@ -545,7 +551,7 @@ int witherworm_demo(struct notcurses* nc){
         return -1;
       }
       if(message(mess, maxy, maxx, i, sizeof(steps) / sizeof(*steps),
-                  bytes_out, egcs_out, cols_out)){
+                 bytes_out, egcs_out, cols_out)){
         ncplane_destroy(math); ncplane_destroy(mess);
         return -1;
       }
