@@ -1,16 +1,16 @@
 #include "internal.h"
 
 void ncplane_greyscale(ncplane *n){
-  for(int y = 0 ; y < n->leny ; ++y){
-    for(int x = 0 ; x < n->lenx ; ++x){
+  for(unsigned y = 0 ; y < n->leny ; ++y){
+    for(unsigned x = 0 ; x < n->lenx ; ++x){
       nccell* c = &n->fb[nfbcellidx(n, y, x)];
       unsigned r, g, b;
-      cell_fg_rgb8(c, &r, &g, &b);
+      nccell_fg_rgb8(c, &r, &g, &b);
       int gy = rgb_greyscale(r, g, b);
-      cell_set_fg_rgb8(c, gy, gy, gy);
-      cell_bg_rgb8(c, &r, &g, &b);
+      nccell_set_fg_rgb8(c, gy, gy, gy);
+      nccell_bg_rgb8(c, &r, &g, &b);
       gy = rgb_greyscale(r, g, b);
-      cell_set_bg_rgb8(c, gy, gy, gy);
+      nccell_set_bg_rgb8(c, gy, gy, gy);
     }
   }
 }
@@ -20,89 +20,121 @@ void ncplane_greyscale(ncplane *n){
 // success. so a return of 0 means there's no work to be done here, and N means
 // we did some work here, filling everything we could reach. out-of-plane is 0.
 static int
-ncplane_polyfill_recurse(ncplane* n, int y, int x, const nccell* c, const char* filltarg){
-  if(y >= n->leny || x >= n->lenx){
-    return 0; // not fillable
+ncplane_polyfill_inner(ncplane* n, unsigned y, unsigned x, const nccell* c, const char* filltarg){
+  struct topolyfill* stack = NULL;
+  if(create_polyfill_op(y, x, &stack) == NULL){
+    return -1;
   }
-  if(y < 0 || x < 0){
-    return 0; // not fillable
-  }
-  nccell* cur = &n->fb[nfbcellidx(n, y, x)];
-  const char* glust = cell_extended_gcluster(n, cur);
+  int ret = 0;
+  struct topolyfill* s;
+  do{
+    s = stack;
+    stack = stack->next;
+    y = s->y;
+    x = s->x;
+    nccell* cur = &n->fb[nfbcellidx(n, y, x)];
+    const char* glust = nccell_extended_gcluster(n, cur);
 //fprintf(stderr, "checking %d/%d (%s) for [%s]\n", y, x, glust, filltarg);
-  if(strcmp(glust, filltarg)){
-    return 0;
-  }
-  if(cell_duplicate(n, cur, c) < 0){
-    return -1;
-  }
-  int r, ret = 1;
+    if(strcmp(glust, filltarg) == 0){
+      ++ret;
+      if(nccell_duplicate(n, cur, c) < 0){
+        goto err;
+      }
 //fprintf(stderr, "blooming from %d/%d ret: %d\n", y, x, ret);
-  if((r = ncplane_polyfill_recurse(n, y - 1, x, c, filltarg)) < 0){
-    return -1;
-  }
-  ret += r;
-  if((r = ncplane_polyfill_recurse(n, y + 1, x, c, filltarg)) < 0){
-    return -1;
-  }
-  ret += r;
-  if((r = ncplane_polyfill_recurse(n, y, x - 1, c, filltarg)) < 0){
-    return -1;
-  }
-  ret += r;
-  if((r = ncplane_polyfill_recurse(n, y, x + 1, c, filltarg)) < 0){
-    return -1;
-  }
-  ret += r;
-  return ret;
-}
-
-// at the initial step only, invalid y, x is an error, so explicitly check.
-int ncplane_polyfill_yx(ncplane* n, int y, int x, const nccell* c){
-  int ret = -1;
-  if(y < n->leny && x < n->lenx){
-    if(y >= 0 && x >= 0){
-      if(y >= n->leny || x >= n->lenx){
-        return -1; // not fillable
+      if(y){
+        if(create_polyfill_op(y - 1, x, &stack) == NULL){
+          goto err;
+        }
       }
-      if(y < 0 || x < 0){
-        return -1; // not fillable
+      if(y + 1 < n->leny){
+        if(create_polyfill_op(y + 1, x, &stack) == NULL){
+          goto err;
+        }
       }
-      const nccell* cur = &n->fb[nfbcellidx(n, y, x)];
-      const char* targ = cell_extended_gcluster(n, cur);
-      const char* fillegc = cell_extended_gcluster(n, c);
-//fprintf(stderr, "checking %d/%d (%s) for [%s]\n", y, x, targ, fillegc);
-      if(strcmp(fillegc, targ) == 0){
-        return 0;
+      if(x){
+        if(create_polyfill_op(y, x - 1, &stack) == NULL){
+          goto err;
+        }
       }
-      // we need an external copy of this, since we'll be writing to it on
-      // the first call into ncplane_polyfill_recurse()
-      char* targcopy = strdup(targ);
-      if(targcopy){
-        ret = ncplane_polyfill_recurse(n, y, x, c, targcopy);
-        free(targcopy);
+      if(x + 1 < n->lenx){
+        if(create_polyfill_op(y, x + 1, &stack) == NULL){
+          goto err;
+        }
       }
     }
+    free(s);
+  }while(stack);
+  return ret;
+
+err:
+  free(s);
+  while(stack){
+    struct topolyfill* tmp = stack->next;
+    free(stack);
+    stack = tmp;
+  }
+  return -1;
+}
+
+// at the initial step only, invalid ystart, xstart is an error, so explicitly check.
+int ncplane_polyfill_yx(ncplane* n, int ystart, int xstart, const nccell* c){
+  if(ystart < 0){
+    if(ystart != -1){
+      logerror("invalid y: %d", ystart);
+      return -1;
+    }
+    ystart = n->y;
+  }
+  unsigned y = ystart;
+  if(xstart < 0){
+    if(xstart != -1){
+      logerror("invalid x: %d", xstart);
+      return -1;
+    }
+    xstart = n->x;
+  }
+  unsigned x = xstart;
+  if(y >= n->leny || x >= n->lenx){
+    logerror("invalid start: %u/%u (%u/%u)", y, x, n->leny, n->lenx);
+    return -1;
+  }
+  const nccell* cur = &n->fb[nfbcellidx(n, y, x)];
+  const char* targ = nccell_extended_gcluster(n, cur);
+  const char* fillegc = nccell_extended_gcluster(n, c);
+//fprintf(stderr, "checking %d/%d (%s) for [%s]\n", y, x, targ, fillegc);
+  if(strcmp(fillegc, targ) == 0){
+    return 0;
+  }
+  int ret = -1;
+  // we need an external copy of this, since we'll be writing to it on
+  // the first call into ncplane_polyfill_inner()
+  char* targcopy = strdup(targ);
+  if(targcopy){
+    ret = ncplane_polyfill_inner(n, y, x, c, targcopy);
+    free(targcopy);
   }
   return ret;
 }
 
 static bool
 check_gradient_channel_args(uint32_t ul, uint32_t ur, uint32_t bl, uint32_t br){
-  if(channel_default_p(ul) || channel_default_p(ur) ||
-     channel_default_p(bl) || channel_default_p(br)){
-    if(!(channel_default_p(ul) && channel_default_p(ur) &&
-         channel_default_p(bl) && channel_default_p(br))){
+  if(ncchannel_default_p(ul) || ncchannel_default_p(ur) ||
+     ncchannel_default_p(bl) || ncchannel_default_p(br)){
+    if(!(ncchannel_default_p(ul) && ncchannel_default_p(ur) &&
+         ncchannel_default_p(bl) && ncchannel_default_p(br))){
+      logerror("some (not all) channels were defaults");
       return true;
     }
   }
-  if(channel_alpha(ul) != channel_alpha(ur) ||
-     channel_alpha(ur) != channel_alpha(bl) ||
-     channel_alpha(bl) != channel_alpha(br)){
+  if(ncchannel_alpha(ul) != ncchannel_alpha(ur) ||
+     ncchannel_alpha(ur) != ncchannel_alpha(bl) ||
+     ncchannel_alpha(bl) != ncchannel_alpha(br)){
+    logerror("channel alphas didn't match");
     return true;
   }
-  if(channel_palindex_p(ul) || channel_palindex_p(bl) ||
-     channel_palindex_p(br) || channel_palindex_p(ur)){
+  if(ncchannel_palindex_p(ul) || ncchannel_palindex_p(bl) ||
+     ncchannel_palindex_p(br) || ncchannel_palindex_p(ur)){
+    logerror("can't blend palette-indexed color");
     return true;
   }
   return false;
@@ -116,12 +148,12 @@ check_gradient_channel_args(uint32_t ul, uint32_t ur, uint32_t bl, uint32_t br){
 // - all backgrounds must have the same alpha
 // - palette-indexed color must not be used
 bool check_gradient_args(uint64_t ul, uint64_t ur, uint64_t bl, uint64_t br){
-  if(check_gradient_channel_args(channels_fchannel(ul), channels_fchannel(ur),
-                                 channels_fchannel(bl), channels_fchannel(br))){
+  if(check_gradient_channel_args(ncchannels_fchannel(ul), ncchannels_fchannel(ur),
+                                 ncchannels_fchannel(bl), ncchannels_fchannel(br))){
     return true;
   }
-  if(check_gradient_channel_args(channels_bchannel(ul), channels_bchannel(ur),
-                                 channels_bchannel(bl), channels_bchannel(br))){
+  if(check_gradient_channel_args(ncchannels_bchannel(ul), ncchannels_bchannel(ur),
+                                 ncchannels_bchannel(bl), ncchannels_bchannel(br))){
     return true;
   }
   return false;
@@ -131,171 +163,129 @@ bool check_gradient_args(uint64_t ul, uint64_t ur, uint64_t bl, uint64_t br){
 // we're using double halfblocks, into `c`->channels.
 static inline void
 calc_highgradient(nccell* c, uint32_t ul, uint32_t ur, uint32_t ll,
-                  uint32_t lr, int y, int x, int ylen, int xlen){
-  if(!channel_default_p(ul)){
+                  uint32_t lr, unsigned y, unsigned x,
+                  unsigned ylen, unsigned xlen){
+  if(!ncchannel_default_p(ul)){
     cell_set_fchannel(c, calc_gradient_channel(ul, ur, ll, lr,
                                                y * 2, x, ylen, xlen));
     cell_set_bchannel(c, calc_gradient_channel(ul, ur, ll, lr,
                                                y * 2 + 1, x, ylen, xlen));
   }else{
-    cell_set_fg_default(c);
-    cell_set_bg_default(c);
+    nccell_set_fg_default(c);
+    nccell_set_bg_default(c);
   }
 }
 
-int ncplane_highgradient(ncplane* n, uint32_t ul, uint32_t ur,
-                         uint32_t ll, uint32_t lr, int ystop, int xstop){
+int ncplane_gradient2x1(ncplane* n, int y, int x, unsigned ylen, unsigned xlen,
+                        uint32_t ul, uint32_t ur, uint32_t ll, uint32_t lr){
   if(!notcurses_canutf8(ncplane_notcurses(n))){
-    logerror(ncplane_notcurses(n), "Highdef gradients require UTF8\n");
+    logerror("highdef gradients require utf8");
     return -1;
   }
   if(check_gradient_channel_args(ul, ur, ll, lr)){
-    logerror(ncplane_notcurses(n), "Invalid highdef gradient channels\n");
     return -1;
   }
-  int yoff, xoff, ymax, xmax;
-  ncplane_cursor_yx(n, &yoff, &xoff);
-  // must be at least 1x1, with its upper-left corner at the current cursor
-  if(ystop < yoff){
+  unsigned ystart, xstart;
+  if(check_geometry_args(n, y, x, &ylen, &xlen, &ystart, &xstart)){
     return -1;
   }
-  if(xstop < xoff){
-    return -1;
-  }
-  ncplane_dim_yx(n, &ymax, &xmax);
-  // must be within the ncplane
-  if(xstop >= xmax || ystop >= ymax){
-    return -1;
-  }
-  const int xlen = xstop - xoff + 1;
-  const int ylen = (ystop - yoff + 1) * 2;
   if(xlen == 1){
     if(ul != ur || ll != lr){
+      logerror("horizontal channel variation in single column");
       return -1;
     }
   }
   int total = 0;
-  for(int y = yoff ; y <= ystop ; ++y){
-    for(int x = xoff ; x <= xstop ; ++x){
-      nccell* targc = ncplane_cell_ref_yx(n, y, x);
+  for(unsigned yy = ystart ; yy < ystart + ylen ; ++yy){
+    for(unsigned xx = xstart ; xx < xstart + xlen ; ++xx){
+      nccell* targc = ncplane_cell_ref_yx(n, yy, xx);
       targc->channels = 0;
       if(pool_blit_direct(&n->pool, targc, "▀", strlen("▀"), 1) <= 0){
         return -1;
       }
-      calc_highgradient(targc, ul, ur, ll, lr, y - yoff, x - xoff, ylen, xlen);
+      calc_highgradient(targc, ul, ur, ll, lr, yy - ystart, xx - xstart, ylen * 2, xlen);
       ++total;
     }
   }
   return total;
 }
 
+// FIXME remove in abi3
 int ncplane_highgradient_sized(ncplane* n, uint32_t ul, uint32_t ur,
                                uint32_t ll, uint32_t lr, int ylen, int xlen){
   if(ylen < 1 || xlen < 1){
     return -1;
   }
-  int y, x;
   if(!notcurses_canutf8(ncplane_notcurses_const(n))){
     // this works because the uin32_ts we pass in will be promoted to uint64_ts
     // via extension, and the space will employ the background. mwahh!
-    return ncplane_gradient_sized(n, " ", 0, ul, ur, ll, lr, ylen, xlen);
+    return ncplane_gradient(n, -1, -1, ylen, xlen, " ", 0, ul, ur, ll, lr);
   }
-  ncplane_cursor_yx(n, &y, &x);
-  return ncplane_highgradient(n, ul, ur, ll, lr, y + ylen - 1, x + xlen - 1);
+  return ncplane_gradient2x1(n, -1, -1, ylen, xlen, ul, ur, ll, lr);
 }
 
-int ncplane_gradient(ncplane* n, const char* egc, uint32_t stylemask,
-                     uint64_t ul, uint64_t ur, uint64_t bl, uint64_t br,
-                     int ystop, int xstop){
+int ncplane_gradient(ncplane* n, int y, int x, unsigned ylen, unsigned xlen,
+                     const char* egc, uint16_t stylemask,
+                     uint64_t ul, uint64_t ur, uint64_t bl, uint64_t br){
   if(check_gradient_args(ul, ur, bl, br)){
-    logerror(ncplane_notcurses(n), "Illegal gradient inputs\n");
     return -1;
   }
-  if(egc == NULL){
+  unsigned ystart, xstart;
+  if(check_geometry_args(n, y, x, &ylen, &xlen, &ystart, &xstart)){
     return -1;
   }
-  int yoff, xoff, ymax, xmax;
-  ncplane_cursor_yx(n, &yoff, &xoff);
-  // must be at least 1x1, with its upper-left corner at the current cursor
-  if(ystop < yoff){
-    logerror(ncplane_notcurses(n), "Ystop %d < yoff %d\n", ystop, yoff);
-    return -1;
-  }
-  if(xstop < xoff){
-    logerror(ncplane_notcurses(n), "Xstop %d < xoff %d\n", xstop, xoff);
-    return -1;
-  }
-  ncplane_dim_yx(n, &ymax, &xmax);
-  // must be within the ncplane
-  if(xstop >= xmax || ystop >= ymax){
-    return -1;
-  }
-  const int xlen = xstop - xoff + 1;
-  const int ylen = ystop - yoff + 1;
   if(ylen == 1){
     if(xlen == 1){
       if(ul != ur || ur != br || br != bl){
+        logerror("channel variation in 1x1 area");
         return -1;
       }
     }else{
       if(ul != bl || ur != br){
+        logerror("vertical channel variation in single row");
         return -1;
       }
     }
   }else if(xlen == 1){
     if(ul != ur || bl != br){
+      logerror("horizontal channel variation in single column");
       return -1;
     }
   }
   int total = 0;
-  for(int y = yoff ; y <= ystop ; ++y){
-    for(int x = xoff ; x <= xstop ; ++x){
-      nccell* targc = ncplane_cell_ref_yx(n, y, x);
+  for(unsigned yy = ystart ; yy < ystart + ylen ; ++yy){
+    for(unsigned xx = xstart ; xx < xstart + xlen ; ++xx){
+      nccell* targc = ncplane_cell_ref_yx(n, yy, xx);
       targc->channels = 0;
-      if(cell_load(n, targc, egc) < 0){
+      if(nccell_load(n, targc, egc) < 0){
         return -1;
       }
       targc->stylemask = stylemask;
       calc_gradient_channels(&targc->channels, ul, ur, bl, br,
-                             y - yoff, x - xoff, ylen, xlen);
+                             yy - ystart, xx - xstart, ylen, xlen);
       ++total;
     }
   }
   return total;
 }
 
-int ncplane_stain(ncplane* n, int ystop, int xstop,
+int ncplane_stain(ncplane* n, int y, int x, unsigned ylen, unsigned xlen,
                   uint64_t tl, uint64_t tr, uint64_t bl, uint64_t br){
   // Can't use default or palette-indexed colors in a gradient
   if(check_gradient_args(tl, tr, bl, br)){
-    logerror(ncplane_notcurses(n), "Illegal staining inputs\n");
     return -1;
   }
-  int yoff, xoff, ymax, xmax;
-  ncplane_cursor_yx(n, &yoff, &xoff);
-  // must be at least 1x1, with its upper-left corner at the current cursor
-  if(ystop < yoff){
-    logerror(ncplane_notcurses(n), "Ystop %d < yoff %d\n", ystop, yoff);
+  unsigned ystart, xstart;
+  if(check_geometry_args(n, y, x, &ylen, &xlen, &ystart, &xstart)){
     return -1;
   }
-  if(xstop < xoff){
-    logerror(ncplane_notcurses(n), "Xstop %d < xoff %d\n", xstop, xoff);
-    return -1;
-  }
-  ncplane_dim_yx(n, &ymax, &xmax);
-  // must be within the ncplane
-  if(xstop >= xmax || ystop >= ymax){
-    return -1;
-  }
-  const int xlen = xstop - xoff + 1;
-  const int ylen = ystop - yoff + 1;
   int total = 0;
-  for(int y = yoff ; y <= ystop ; ++y){
-    for(int x = xoff ; x <= xstop ; ++x){
-      nccell* targc = ncplane_cell_ref_yx(n, y, x);
+  for(unsigned yy = ystart ; yy < ystart + ylen ; ++yy){
+    for(unsigned xx = xstart ; xx < xstart + xlen ; ++xx){
+      nccell* targc = ncplane_cell_ref_yx(n, yy, xx);
       if(targc->gcluster){
         calc_gradient_channels(&targc->channels, tl, tr, bl, br,
-                               y - yoff, x - xoff, ylen, xlen);
+                               yy - ystart, xx - xstart, ylen, xlen);
       }
       ++total;
     }
@@ -303,25 +293,16 @@ int ncplane_stain(ncplane* n, int ystop, int xstop,
   return total;
 }
 
-int ncplane_format(ncplane* n, int ystop, int xstop, uint32_t stylemask){
-  int yoff, xoff, ymax, xmax;
-  ncplane_cursor_yx(n, &yoff, &xoff);
-  // must be at least 1x1, with its upper-left corner at the current cursor
-  if(ystop < yoff){
-    return -1;
-  }
-  if(xstop < xoff){
-    return -1;
-  }
-  ncplane_dim_yx(n, &ymax, &xmax);
-  // must be within the ncplane
-  if(xstop >= xmax || ystop >= ymax){
+int ncplane_format(ncplane* n, int y, int x, unsigned ylen,
+                   unsigned xlen, uint16_t stylemask){
+  unsigned ystart, xstart;
+  if(check_geometry_args(n, y, x, &ylen, &xlen, &ystart, &xstart)){
     return -1;
   }
   int total = 0;
-  for(int y = yoff ; y < ystop + 1 ; ++y){
-    for(int x = xoff ; x < xstop + 1 ; ++x){
-      nccell* targc = ncplane_cell_ref_yx(n, y, x);
+  for(unsigned yy = ystart ; yy < ystart + ylen ; ++yy){
+    for(unsigned xx = xstart ; xx < xstart + xlen ; ++xx){
+      nccell* targc = ncplane_cell_ref_yx(n, yy, xx);
       targc->stylemask = stylemask;
       ++total;
     }
@@ -333,7 +314,7 @@ int ncplane_format(ncplane* n, int ystop, int xstop, uint32_t stylemask){
 // the background. if we're a full block, set both to the foreground.
 static int
 rotate_channels(ncplane* src, const nccell* c, uint32_t* fchan, uint32_t* bchan){
-  const char* egc = cell_extended_gcluster(src, c);
+  const char* egc = nccell_extended_gcluster(src, c);
   if(egc[0] == ' ' || egc[0] == 0){
     *fchan = *bchan;
     return 0;
@@ -346,19 +327,19 @@ rotate_channels(ncplane* src, const nccell* c, uint32_t* fchan, uint32_t* bchan)
     *bchan = *fchan;
     return 0;
   }
-  logerror(ncplane_notcurses(src), "Invalid EGC for rotation [%s]\n", egc);
+  logerror("invalid EGC for rotation [%s]", egc);
   return -1;
 }
 
 static int
 rotate_output(ncplane* dst, uint32_t tchan, uint32_t bchan){
-  dst->channels = channels_combine(tchan, bchan);
+  dst->channels = ncchannels_combine(tchan, bchan);
   if(tchan != bchan){
     return ncplane_putegc(dst, "▀", NULL);
   }
-  if(channel_default_p(tchan) && channel_default_p(bchan)){
+  if(ncchannel_default_p(tchan) && ncchannel_default_p(bchan)){
     return ncplane_putegc(dst, "", NULL);
-  }else if(channel_default_p(tchan)){
+  }else if(ncchannel_default_p(tchan)){
     return ncplane_putegc(dst, " ", NULL);
   }
   return ncplane_putegc(dst, "█", NULL);
@@ -393,13 +374,13 @@ rotate_output(ncplane* dst, uint32_t tchan, uint32_t bchan){
 //   lower?) having the two channels as fore- and background.
 static int
 rotate_2x1_cw(ncplane* src, ncplane* dst, int srcy, int srcx, int dsty, int dstx){
-  nccell c1 = CELL_TRIVIAL_INITIALIZER;
-  nccell c2 = CELL_TRIVIAL_INITIALIZER;
+  nccell c1 = NCCELL_TRIVIAL_INITIALIZER;
+  nccell c2 = NCCELL_TRIVIAL_INITIALIZER;
   if(ncplane_at_yx_cell(src, srcy, srcx, &c1) < 0){
     return -1;
   }
   if(ncplane_at_yx_cell(src, srcy, srcx + 1, &c2) < 0){
-    cell_release(src, &c1);
+    nccell_release(src, &c1);
     return -1;
   }
   // there can be at most 4 colors and 4 transparencies:
@@ -434,13 +415,13 @@ rotate_2x1_cw(ncplane* src, ncplane* dst, int srcy, int srcx, int dsty, int dstx
 
 static int
 rotate_2x1_ccw(ncplane* src, ncplane* dst, int srcy, int srcx, int dsty, int dstx){
-  nccell c1 = CELL_TRIVIAL_INITIALIZER;
-  nccell c2 = CELL_TRIVIAL_INITIALIZER;
+  nccell c1 = NCCELL_TRIVIAL_INITIALIZER;
+  nccell c2 = NCCELL_TRIVIAL_INITIALIZER;
   if(ncplane_at_yx_cell(src, srcy, srcx, &c1) < 0){
     return -1;
   }
   if(ncplane_at_yx_cell(src, srcy, srcx + 1, &c2) < 0){
-    cell_release(src, &c1);
+    nccell_release(src, &c1);
     return -1;
   }
   uint32_t c1b = cell_bchannel_common(&c1);
@@ -459,12 +440,12 @@ rotate_2x1_ccw(ncplane* src, ncplane* dst, int srcy, int srcx, int dsty, int dst
 // copy 'newp' into 'n' after resizing 'n' to match 'newp'
 static int
 rotate_merge(ncplane* n, ncplane* newp){
-  int dimy, dimx;
+  unsigned dimy, dimx;
   ncplane_dim_yx(newp, &dimy, &dimx);
   int ret = ncplane_resize(n, 0, 0, 0, 0, 0, 0, dimy, dimx);
   if(ret == 0){
-    for(int y = 0 ; y < dimy ; ++y){
-      for(int x = 0 ; x < dimx ; ++x){
+    for(unsigned y = 0 ; y < dimy ; ++y){
+      for(unsigned x = 0 ; x < dimx ; ++x){
         const nccell* src = &newp->fb[fbcellidx(y, dimx, x)];
         nccell* targ = &n->fb[fbcellidx(y, dimx, x)];
         if(cell_duplicate_far(&n->pool, targ, newp, src) < 0){
@@ -481,7 +462,7 @@ static ncplane*
 rotate_plane(ncplane* n){
   int absy, absx;
   ncplane_yx(n, &absy, &absx);
-  int dimy, dimx;
+  unsigned dimy, dimx;
   ncplane_dim_yx(n, &dimy, &dimx);
   if(dimx % 2 != 0){
     return NULL;
@@ -505,7 +486,7 @@ int ncplane_rotate_cw(ncplane* n){
   if(newp == NULL){
     return -1;
   }
-  int dimy, dimx;
+  unsigned dimy, dimx;
   ncplane_dim_yx(n, &dimy, &dimx);
   int centy, centx;
   ncplane_center_abs(n, &centy, &centx);
@@ -515,9 +496,9 @@ int ncplane_rotate_cw(ncplane* n){
   // the top two leftmost cells. work from the bottom up on the source, so we
   // can copy to the top row from the left to the right.
   int targx, targy = 0;
-  for(int x = 0 ; x < dimx ; x += 2){
+  for(unsigned x = 0 ; x < dimx ; x += 2){
     targx = 0;
-    for(int y = dimy - 1 ; y >= 0 ; --y){
+    for(int y = (int)dimy - 1 ; y >= 0 ; --y){
       if(rotate_2x1_cw(n, newp, y, x, targy, targx)){
         ncplane_destroy(newp);
         return -1;
@@ -536,15 +517,16 @@ int ncplane_rotate_ccw(ncplane* n){
   if(newp == NULL){
     return -1;
   }
-  int dimy, dimx, targdimy, targdimx;
+  unsigned dimy, dimx, targdimy, targdimx;
   ncplane_dim_yx(n, &dimy, &dimx);
   ncplane_dim_yx(newp, &targdimy, &targdimx);
-  int x = dimx - 2, y;
+  int x = (int)dimx - 2;
+  int y;
   // Each row of the target plane is taken from a column of the source plane.
   // As the target row grows (down), the source column shrinks (moves left).
-  for(int targy = 0 ; targy < targdimy ; ++targy){
+  for(unsigned targy = 0 ; targy < targdimy ; ++targy){
     y = 0;
-    for(int targx = 0 ; targx < targdimx ; targx += 2){
+    for(unsigned targx = 0 ; targx < targdimx ; targx += 2){
       if(rotate_2x1_ccw(n, newp, y, x, targy, targx)){
         ncplane_destroy(newp);
         return -1;
@@ -563,18 +545,18 @@ int ncplane_rotate_ccw(ncplane* n){
 #define QR_BASE_SIZE 17
 #define PER_QR_VERSION 4
 
-static inline int
-qrcode_rows(int version){
+static inline unsigned
+qrcode_rows(unsigned version){
   return QR_BASE_SIZE + (version * PER_QR_VERSION);
 }
 
-static inline int
-qrcode_cols(int version){
+static inline unsigned
+qrcode_cols(unsigned version){
   return QR_BASE_SIZE + (version * PER_QR_VERSION);
 }
 
-int ncplane_qrcode(ncplane* n, int* ymax, int* xmax, const void* data, size_t len){
-  const ncblitter_e blitter = NCBLIT_2x1;
+int ncplane_qrcode(ncplane* n, unsigned* ymax, unsigned* xmax, const void* data, size_t len){
+  const ncblitter_e blitfxn = NCBLIT_2x1;
   const int MAX_QR_VERSION = 40; // QR library only supports up to 40
   if(*ymax <= 0 || *xmax <= 0){
     return -1;
@@ -639,12 +621,15 @@ int ncplane_qrcode(ncplane* n, int* ymax, int* xmax, const void* data, size_t le
         ret = square;
         struct ncvisual_options vopts = {
           .n = n,
-          .blitter = blitter,
+          .blitter = blitfxn,
         };
-        if(ncvisual_render(ncplane_notcurses(n), ncv, &vopts) == n){
+        if(ncvisual_blit(ncplane_notcurses(n), ncv, &vopts) == n){
           ret = square;
         }
-        ncvisual_geom(ncplane_notcurses(n), ncv, &vopts, NULL, NULL, &yscale, &xscale);
+        ncvgeom geom;
+        ncvisual_geom(ncplane_notcurses(n), NULL, &vopts, &geom);
+        yscale = geom.scaley;
+        xscale = geom.scalex;
       }
       ncvisual_destroy(ncv);
     }
@@ -660,7 +645,7 @@ int ncplane_qrcode(ncplane* n, int* ymax, int* xmax, const void* data, size_t le
   return -1;
 }
 #else
-int ncplane_qrcode(ncplane* n, int* ymax, int* xmax, const void* data, size_t len){
+int ncplane_qrcode(ncplane* n, unsigned* ymax, unsigned* xmax, const void* data, size_t len){
   (void)n;
   (void)ymax;
   (void)xmax;
