@@ -8,6 +8,7 @@
 // common elements of type-parameterized plots
 typedef struct ncplot {
   ncplane* ncp;
+  ncplane* pixelp; // only used for NCBLIT_PIXEL
   /* sloutcount-element circular buffer of samples. the newest one (rightmost)
      is at slots[slotstart]; they get older as you go back (and around).
      elements. slotcount is max(columns, rangex), less label room. */
@@ -16,25 +17,70 @@ typedef struct ncplot {
   uint64_t minchannels;
   uint16_t legendstyle;
   bool vertical_indep; /* not yet implemented FIXME */
+  unsigned chancount; // channel count (can change on cell-pixel geom change)
+  uint64_t* channels; // computed in calculate_gradient_vector() (constructor)
   const struct blitset* bset;
   char* title;
   /* requested number of slots. 0 for automatically setting the number of slots
      to span the horizontal area. if there are more slots than there are
      columns, we prefer showing more recent slots to less recent. if there are
      fewer slots than there are columns, they prefer the left side. */
-  int rangex;
+  unsigned rangex;
   /* domain minimum and maximum. if detectdomain is true, these are
      progressively enlarged/shrunk to fit the sample set. if not, samples
      outside these bounds are counted, but the displayed range covers only this. */
-  int slotcount;
+  unsigned slotcount;
   int slotstart; /* index of most recently-written slot */
-  bool labelaxisd; /* label dependent axis (consumes PREFIXCOLUMNS columns) */
+  bool labelaxisd; /* label dependent axis (consumes NCPREFIXCOLUMNS columns) */
   bool exponentiali; /* exponential independent axis */
   bool detectdomain; /* is domain detection in effect (stretch the domain)? */
   bool detectonlymax; /* domain detection applies only to max, not min */
   bool printsample; /* print the most recent sample */
 } ncplot;
 
+static inline int
+create_pixelp(ncplot *p, ncplane* n){
+  if(((p->pixelp = ncplane_dup(n, NULL)) == NULL)){
+    return -1;
+  }
+  if(ncplane_set_name(p->pixelp, "pmap")){
+    ncplane_destroy(p->pixelp);
+    return -1;
+  }
+  ncplane_reparent(p->pixelp, n);
+  ncplane_move_below(p->pixelp, n);
+  uint64_t basechan = 0;
+  ncchannels_set_bg_alpha(&basechan, NCALPHA_TRANSPARENT);
+  ncchannels_set_fg_alpha(&basechan, NCALPHA_TRANSPARENT);
+  ncplane_set_base(n, "", 0, basechan);
+  return 0;
+}
+
+// we have some color gradient across the life of the plot (almost; it gets
+// recalculated if the cell-pixel geometry changes and we're using
+// NCBLIT_PIXEL). if we're using cell blitting, we only get one channel pair
+// per row, no matter what height we have. with pixels, we get cellpxy * rows.
+static int
+calculate_gradient_vector(ncplot* p, unsigned pixelp){
+  const int dimy = ncplane_dim_y(p->ncp);
+  const unsigned states = dimy * (pixelp ? ncplane_pile(p->ncp)->cellpxy : 1);
+  if(states == p->chancount){ // no need to recalculate
+    return 0;
+  }
+  uint64_t* tmp = realloc(p->channels, states * sizeof(*p->channels));
+  if(tmp == NULL){
+    return -1;
+  }
+  p->channels = tmp;
+  p->chancount = states;
+  for(unsigned y = 0 ; y < p->chancount ; ++y){ \
+    calc_gradient_channels(&p->channels[y], p->minchannels, p->minchannels,
+                           p->maxchannels, p->maxchannels,
+                           y, 0, p->chancount, 0);
+  }
+  return 0;
+}
+
 #define MAXWIDTH 2
 #define CREATE(T, X) \
 typedef struct nc##X##plot { \
@@ -43,17 +89,20 @@ typedef struct nc##X##plot { \
   ncplot plot; \
 } nc##X##plot; \
 \
-int redraw_plot_##T(nc##X##plot* ncp){ \
+int redraw_pixelplot_##T(nc##X##plot* ncp){ \
+  if(calculate_gradient_vector(&ncp->plot, 1)){ \
+    return -1; \
+  } \
+  const int scale = ncplane_pile_const(ncp->plot.ncp)->cellpxx; \
   ncplane_erase(ncp->plot.ncp); \
-  const int scale = ncp->plot.bset->width; \
-  int dimy, dimx; \
+  unsigned dimy, dimx; \
   ncplane_dim_yx(ncp->plot.ncp, &dimy, &dimx); \
-  const int scaleddim = dimx * scale; \
+  const unsigned scaleddim = dimx * scale; \
   /* each transition is worth this much change in value */ \
-  const size_t states = ncp->plot.bset->height + 1; \
+  const size_t states = ncplane_pile_const(ncp->plot.ncp)->cellpxy; \
   /* FIXME can we not rid ourselves of this meddlesome double? either way, the \
-     interval is one row's range (for linear plots), or the base (base^slots== \
-     maxy-miny) of the range (for exponential plots). */ \
+     interval is one row's range (for linear plots), or the base \
+     (base^slots == maxy-miny) of the range (for exponential plots). */ \
   double interval; \
   if(ncp->plot.exponentiali){ \
     if(ncp->maxy > ncp->miny){ \
@@ -65,44 +114,208 @@ int redraw_plot_##T(nc##X##plot* ncp){ \
   }else{ \
     interval = ncp->maxy < ncp->miny ? 0 : (ncp->maxy - ncp->miny) / ((double)dimy * states); \
   } \
-  const int startx = ncp->plot.labelaxisd ? PREFIXCOLUMNS : 0; /* plot cols begin here */ \
+  const int startx = ncp->plot.labelaxisd ? NCPREFIXCOLUMNS : 0; /* plot cols begin here */ \
   /* if we want fewer slots than there are available columns, our final column \
      will be other than the plane's final column. most recent x goes here. */ \
-  const int finalx = (ncp->plot.slotcount < scaleddim - 1 - (startx * scale) ? \
-                     startx + (ncp->plot.slotcount / scale) - 1 : dimx - 1); \
+  const unsigned finalx = (ncp->plot.slotcount < scaleddim - 1 - (startx * scale) ? \
+                          startx + (ncp->plot.slotcount / scale) - 1 : dimx - 1); \
   ncplane_set_styles(ncp->plot.ncp, ncp->plot.legendstyle); \
   if(ncp->plot.labelaxisd){ \
     /* show the *top* of each interval range */ \
-    for(int y = 0 ; y < dimy ; ++y){ \
-      uint64_t channels = 0; \
-      calc_gradient_channels(&channels, ncp->plot.minchannels, ncp->plot.minchannels, \
-                             ncp->plot.maxchannels, ncp->plot.maxchannels, y, 0, dimy, dimx); \
-      ncplane_set_channels(ncp->plot.ncp, channels); \
-      char buf[PREFIXSTRLEN + 1]; \
+    for(unsigned y = 0 ; y < dimy ; ++y){ \
+      ncplane_set_channels(ncp->plot.ncp, ncp->plot.channels[y * states]); \
+      char buf[NCPREFIXSTRLEN + 1]; \
       if(ncp->plot.exponentiali){ \
         if(y == dimy - 1){ /* we cheat on the top row to exactly match maxy */ \
-          ncmetric(ncp->maxy * 100, 100, buf, 0, 1000, '\0'); \
+          ncqprefix(ncp->maxy * 100, 100, buf, 0); \
         }else{ \
-          ncmetric(pow(interval, (y + 1) * states) * 100, 100, buf, 0, 1000, '\0'); \
+          ncqprefix(pow(interval, (y + 1) * states) * 100, 100, buf, 0); \
         } \
       }else{ \
-        ncmetric((ncp->maxy - interval * states * (dimy - y - 1)) * 100, 100, buf, 0, 1000, '\0'); \
+        ncqprefix((ncp->maxy - interval * states * (dimy - y - 1)) * 100, 100, buf, 0); \
       } \
       if(y == dimy - 1 && strlen(ncp->plot.title)){ \
-        ncplane_printf_yx(ncp->plot.ncp, dimy - y - 1, PREFIXCOLUMNS - strlen(buf), "%s %s", buf, ncp->plot.title); \
+        ncplane_printf_yx(ncp->plot.ncp, dimy - y - 1, 0, "%*.*s %s", \
+                          NCPREFIXSTRLEN, NCPREFIXSTRLEN, buf, ncp->plot.title); \
       }else{ \
-        ncplane_printf_yx(ncp->plot.ncp, dimy - y - 1, PREFIXCOLUMNS - strlen(buf), "%s", buf); \
+        ncplane_printf_yx(ncp->plot.ncp, dimy - y - 1, 0, "%*.*s", \
+                          NCPREFIXSTRLEN, NCPREFIXSTRLEN, buf); \
       } \
     } \
   }else if(strlen(ncp->plot.title)){ \
-    uint64_t channels = 0; \
-    calc_gradient_channels(&channels, ncp->plot.minchannels, ncp->plot.minchannels, \
-                           ncp->plot.maxchannels, ncp->plot.maxchannels, dimy - 1, 0, dimy, dimx); \
-    ncplane_set_channels(ncp->plot.ncp, channels); \
-    ncplane_printf_yx(ncp->plot.ncp, 0, PREFIXCOLUMNS - strlen(ncp->plot.title), "%s", ncp->plot.title); \
+    ncplane_set_channels(ncp->plot.ncp, ncp->plot.channels[(dimy - 1) * states]); \
+    ncplane_printf_yx(ncp->plot.ncp, 0, NCPREFIXCOLUMNS - strlen(ncp->plot.title), "%s", ncp->plot.title); \
   } \
   ncplane_set_styles(ncp->plot.ncp, NCSTYLE_NONE); \
-  if(finalx < startx){ /* exit on pathologically narrow planes */ \
+  if((int)finalx < startx){ /* exit on pathologically narrow planes */ \
+    return 0; \
+  } \
+  if(!interval){ \
+    interval = 1; \
+  } \
+  uint32_t* pixels = malloc(dimy * dimx * states * scale * sizeof(*pixels)); \
+  if(pixels == NULL){ \
+    return -1; \
+  } \
+  /* FIXME just zero out as we copy to each */ \
+  memset(pixels, 0, dimy * dimx * states * scale * sizeof(*pixels)); \
+  /* a column corresponds to |scale| slots' worth of samples. prepare the working gval set. */ \
+  T* gvals = malloc(sizeof(*gvals) * scale); \
+  if(gvals == NULL){ \
+    free(pixels); \
+    return -1; \
+  } \
+  int idx = ncp->plot.slotstart; /* idx holds the real slot index; we move backwards */ \
+  /* iterate backwards across the plot from the final (rightmost) x being \
+     plotted (finalx) to the first (leftmost) x being plotted (startx).   */ \
+  for(int x = finalx ; x >= startx ; --x){ \
+    /* load gvals retaining the same ordering we have in the actual array */ \
+    for(int i = scale - 1 ; i >= 0 ; --i){ \
+      gvals[i] = ncp->slots[idx]; /* clip the value at the limits of the graph */ \
+      if(gvals[i] < ncp->miny){ \
+        gvals[i] = ncp->miny; \
+      } \
+      if(gvals[i] > ncp->maxy){ \
+        gvals[i] = ncp->maxy; \
+      } \
+      /* FIXME if there are an odd number, only go up through the valid ones... */ \
+      if(--idx < 0){ \
+        idx = ncp->plot.slotcount - 1; \
+      } \
+    } \
+    /* starting from the least-significant row, progress in the more significant \
+       direction, prepping pixels, aborting early if we can't draw anything in a \
+       given cell. */ \
+    T intervalbase = ncp->miny; \
+    bool done = !ncp->plot.bset->fill; \
+    for(unsigned y = 0 ; y < dimy ; ++y){ \
+      /* if we've got at least one interval's worth on the number of positions \
+        times the number of intervals per position plus the starting offset, \
+        we're going to print *something* */ \
+      for(int i = 0 ; i < scale ; ++i){ \
+        size_t egcidx; \
+        if(intervalbase < gvals[i]){ \
+          if(ncp->plot.exponentiali){ \
+            /* we want the log-base-interval of gvals[i] */ \
+            double scaled = log(gvals[i] - ncp->miny) / log(interval); \
+            double sival = intervalbase ? log(intervalbase) / log(interval) : 0; \
+            egcidx = scaled - sival; \
+          }else{ \
+            egcidx = (gvals[i] - intervalbase) / interval; \
+          } \
+          if(egcidx >= states){ \
+            egcidx = states; \
+            done = false; \
+          } \
+        }else{ \
+          egcidx = 0; \
+        } \
+/*fprintf(stderr, "WRITING TO y/x %d/%d (%zu)\n", y, x, dimx * dimy * scale * states); */\
+        for(size_t yy = 0 ; yy < egcidx ; ++yy){ \
+          int poff = x * scale + i + (((dimy - 1 - y) * states + (states - 1 - yy)) * dimx * scale); \
+          uint32_t color = ncchannels_fg_rgb(ncp->plot.channels[y * states + yy]); \
+          ncpixel_set_a(&color, 0xff); \
+          pixels[poff] = color; \
+        } \
+      } \
+      if(done){ \
+        break; \
+      } \
+      if(ncp->plot.exponentiali){ \
+        intervalbase = ncp->miny + pow(interval, (y + 1) * states - 1); \
+      }else{ \
+        intervalbase += (states * interval); \
+      } \
+    } \
+  } \
+  if(ncp->plot.printsample){ \
+    ncplane_set_styles(ncp->plot.ncp, ncp->plot.legendstyle); \
+    ncplane_set_channels(ncp->plot.ncp, ncp->plot.maxchannels); \
+    /* FIXME is this correct for double? */ \
+    /* we use idx, and thus get an immediate count, changing as we load it.
+     * if you want a stable summary, print the previous slot */ \
+    ncplane_printf_aligned(ncp->plot.ncp, 0, NCALIGN_RIGHT, "%" PRIu64, (uint64_t)ncp->slots[idx]); \
+  } \
+  ncplane_home(ncp->plot.ncp); \
+  struct ncvisual* ncv = ncvisual_from_rgba(pixels, dimy * states, dimx * scale * 4, dimx * scale); \
+  free(pixels); \
+  free(gvals); \
+  if(ncv == NULL){ \
+    return -1; \
+  } \
+  struct ncvisual_options vopts = { \
+    .n = ncp->plot.pixelp, \
+    .blitter = NCBLIT_PIXEL, \
+    .flags = NCVISUAL_OPTION_NODEGRADE, \
+  }; \
+  if(ncvisual_blit(ncplane_notcurses(ncp->plot.ncp), ncv, &vopts) == NULL){ \
+    ncvisual_destroy(ncv); \
+    return -1; \
+  } \
+  ncvisual_destroy(ncv); \
+  return 0; \
+} \
+\
+int redraw_plot_##T(nc##X##plot* ncp){ \
+  if(ncp->plot.bset->geom == NCBLIT_PIXEL){ \
+    return redraw_pixelplot_##T(ncp); \
+  } \
+  if(calculate_gradient_vector(&ncp->plot, 0)){ \
+    return -1; \
+  } \
+  ncplane_erase(ncp->plot.ncp); \
+  const unsigned scale = ncp->plot.bset->width; \
+  unsigned dimy, dimx; \
+  ncplane_dim_yx(ncp->plot.ncp, &dimy, &dimx); \
+  const unsigned scaleddim = dimx * scale; \
+  /* each transition is worth this much change in value */ \
+  const size_t states = ncp->plot.bset->height + 1; \
+  /* FIXME can we not rid ourselves of this meddlesome double? either way, the \
+     interval is one row's range (for linear plots), or the base \
+     (base^slots == maxy-miny) of the range (for exponential plots). */ \
+  double interval; \
+  if(ncp->plot.exponentiali){ \
+    if(ncp->maxy > ncp->miny){ \
+      interval = pow(ncp->maxy - ncp->miny, (double)1 / (dimy * states)); \
+/* fprintf(stderr, "miny: %ju maxy: %ju dimy: %d states: %zu\n", miny, maxy, dimy, states); */ \
+    }else{ \
+      interval = 0; \
+    } \
+  }else{ \
+    interval = ncp->maxy < ncp->miny ? 0 : (ncp->maxy - ncp->miny) / ((double)dimy * states); \
+  } \
+  const int startx = ncp->plot.labelaxisd ? NCPREFIXCOLUMNS : 0; /* plot cols begin here */ \
+  /* if we want fewer slots than there are available columns, our final column \
+     will be other than the plane's final column. most recent x goes here. */ \
+  const unsigned finalx = (ncp->plot.slotcount < scaleddim - 1 - (startx * scale) ? \
+                          startx + (ncp->plot.slotcount / scale) - 1 : dimx - 1); \
+  ncplane_set_styles(ncp->plot.ncp, ncp->plot.legendstyle); \
+  if(ncp->plot.labelaxisd){ \
+    /* show the *top* of each interval range */ \
+    for(unsigned y = 0 ; y < dimy ; ++y){ \
+      ncplane_set_channels(ncp->plot.ncp, ncp->plot.channels[y]); \
+      char buf[NCPREFIXSTRLEN + 1]; \
+      if(ncp->plot.exponentiali){ \
+        if(y == dimy - 1){ /* we cheat on the top row to exactly match maxy */ \
+          ncqprefix(ncp->maxy * 100, 100, buf, 0); \
+        }else{ \
+          ncqprefix(pow(interval, (y + 1) * states) * 100, 100, buf, 0); \
+        } \
+      }else{ \
+        ncqprefix((ncp->maxy - interval * states * (dimy - y - 1)) * 100, 100, buf, 0); \
+      } \
+      if(y == dimy - 1 && strlen(ncp->plot.title)){ \
+        ncplane_printf_yx(ncp->plot.ncp, dimy - y - 1, NCPREFIXCOLUMNS - strlen(buf), "%s %s", buf, ncp->plot.title); \
+      }else{ \
+        ncplane_printf_yx(ncp->plot.ncp, dimy - y - 1, NCPREFIXCOLUMNS - strlen(buf), "%s", buf); \
+      } \
+    } \
+  }else if(strlen(ncp->plot.title)){ \
+    ncplane_set_channels(ncp->plot.ncp, ncp->plot.channels[dimy - 1]); \
+    ncplane_printf_yx(ncp->plot.ncp, 0, NCPREFIXCOLUMNS - strlen(ncp->plot.title), "%s", ncp->plot.title); \
+  } \
+  ncplane_set_styles(ncp->plot.ncp, NCSTYLE_NONE); \
+  if((int)finalx < startx){ /* exit on pathologically narrow planes */ \
     return 0; \
   } \
   if(!interval){ \
@@ -111,7 +324,7 @@ int redraw_plot_##T(nc##X##plot* ncp){ \
   int idx = ncp->plot.slotstart; /* idx holds the real slot index; we move backwards */ \
   for(int x = finalx ; x >= startx ; --x){ \
     /* a single column might correspond to more than 1 ('scale', up to \
-       MAXWIDTH) slot's worth of samples. prepare the working gval set. */ \
+       MAXWIDTH) slots' worth of samples. prepare the working gval set. */ \
     T gvals[MAXWIDTH]; \
     /* load it retaining the same ordering we have in the actual array */ \
     for(int i = scale - 1 ; i >= 0 ; --i){ \
@@ -133,16 +346,13 @@ int redraw_plot_##T(nc##X##plot* ncp){ \
     T intervalbase = ncp->miny; \
     const wchar_t* egc = ncp->plot.bset->plotegcs; \
     bool done = !ncp->plot.bset->fill; \
-    for(int y = 0 ; y < dimy ; ++y){ \
-      uint64_t channels = 0; \
-      calc_gradient_channels(&channels, ncp->plot.minchannels, ncp->plot.minchannels, \
-                             ncp->plot.maxchannels, ncp->plot.maxchannels, y, x, dimy, dimx); \
-      ncplane_set_channels(ncp->plot.ncp, channels); \
+    for(unsigned y = 0 ; y < dimy ; ++y){ \
+      ncplane_set_channels(ncp->plot.ncp, ncp->plot.channels[y]); \
       size_t egcidx = 0, sumidx = 0; \
       /* if we've got at least one interval's worth on the number of positions \
-         times the number of intervals per position plus the starting offset, \
-         we're going to print *something* */ \
-      for(int i = 0 ; i < scale ; ++i){ \
+        times the number of intervals per position plus the starting offset, \
+        we're going to print *something* */ \
+      for(unsigned i = 0 ; i < scale ; ++i){ \
         sumidx *= states; \
         if(intervalbase < gvals[i]){ \
           if(ncp->plot.exponentiali){ \
@@ -164,38 +374,39 @@ int redraw_plot_##T(nc##X##plot* ncp){ \
 /* printf(stderr, "y: %d i(scale): %d gvals[%d]: %ju egcidx: %zu sumidx: %zu interval: %f intervalbase: %ju\n", y, i, i, gvals[i], egcidx, sumidx, interval, intervalbase); */ \
       } \
       /* if we're not UTF8, we can only arrive here via NCBLIT_1x1 (otherwise \
-         we would have errored out during construction). even then, however, \
-         we need handle ASCII differently, since it can't print full block. \
-         in ASCII mode, sumidx != 0 means swap colors and use space. in all \
-         modes, sumidx == 0 means don't do shit, since we erased earlier. */ \
+        we would have errored out during construction). even then, however, \
+        we need handle ASCII differently, since it can't print full block. \
+        in ASCII mode, sumidx != 0 means swap colors and use space. in all \
+        modes, sumidx == 0 means don't do shit, since we erased earlier. */ \
 /* if(sumidx)fprintf(stderr, "dimy: %d y: %d x: %d sumidx: %zu egc[%zu]: %lc\n", dimy, y, x, sumidx, sumidx, egc[sumidx]); */ \
       if(sumidx){ \
+        uint64_t chan = ncp->plot.channels[y]; \
         if(notcurses_canutf8(ncplane_notcurses(ncp->plot.ncp))){ \
-          char utf8[MB_CUR_MAX + 1]; \
+          char utf8[MB_LEN_MAX + 1]; \
           int bytes = wctomb(utf8, egc[sumidx]); \
           if(bytes < 0){ \
             return -1; \
           } \
           utf8[bytes] = '\0'; \
           nccell* c = ncplane_cell_ref_yx(ncp->plot.ncp, dimy - y - 1, x); \
-          cell_set_bchannel(c, ncchannels_bchannel(channels)); \
-          cell_set_fchannel(c, ncchannels_fchannel(channels)); \
+          cell_set_bchannel(c, ncchannels_bchannel(chan)); \
+          cell_set_fchannel(c, ncchannels_fchannel(chan)); \
           nccell_set_styles(c, NCSTYLE_NONE); \
           if(pool_blit_direct(&ncp->plot.ncp->pool, c, utf8, bytes, 1) <= 0){ \
             return -1; \
           } \
         }else{ \
-          const uint64_t swapbg = ncchannels_bchannel(channels); \
-          const uint64_t swapfg = ncchannels_fchannel(channels); \
-          ncchannels_set_bchannel(&channels, swapfg); \
-          ncchannels_set_fchannel(&channels, swapbg); \
-          ncplane_set_channels(ncp->plot.ncp, channels); \
+          const uint64_t swapbg = ncchannels_bchannel(chan); \
+          const uint64_t swapfg = ncchannels_fchannel(chan); \
+          ncchannels_set_bchannel(&chan, swapfg); \
+          ncchannels_set_fchannel(&chan, swapbg); \
+          ncplane_set_channels(ncp->plot.ncp, chan); \
           if(ncplane_putchar_yx(ncp->plot.ncp, dimy - y - 1, x, ' ') <= 0){ \
             return -1; \
           } \
-          ncchannels_set_bchannel(&channels, swapbg); \
-          ncchannels_set_fchannel(&channels, swapfg); \
-          ncplane_set_channels(ncp->plot.ncp, channels); \
+          ncchannels_set_bchannel(&chan, swapbg); \
+          ncchannels_set_fchannel(&chan, swapfg); \
+          ncplane_set_channels(ncp->plot.ncp, chan); \
         } \
       } \
       if(done){ \
@@ -209,44 +420,45 @@ int redraw_plot_##T(nc##X##plot* ncp){ \
     } \
   } \
   if(ncp->plot.printsample){ \
-    int lastslot = ncp->plot.slotstart ? ncp->plot.slotstart - 1 : ncp->plot.slotcount - 1; \
     ncplane_set_styles(ncp->plot.ncp, ncp->plot.legendstyle); \
     ncplane_set_channels(ncp->plot.ncp, ncp->plot.maxchannels); \
-    ncplane_printf_aligned(ncp->plot.ncp, 0, NCALIGN_RIGHT, "%" PRIu64, (uint64_t)ncp->slots[lastslot]); \
+    ncplane_printf_aligned(ncp->plot.ncp, 0, NCALIGN_RIGHT, "%" PRIu64, (uint64_t)ncp->slots[idx]); \
   } \
   ncplane_home(ncp->plot.ncp); \
   return 0; \
 } \
 \
-static bool \
+static const struct blitset* \
 create_##T(nc##X##plot* ncpp, ncplane* n, const ncplot_options* opts, const T miny, const T maxy, \
            const T trueminy, const T truemaxy){ \
-  ncplot_options zeroed = {}; \
+  /* set up ->plot.ncp first so it gets destroyed on error */ \
+  ncpp->plot.ncp = n; \
+  if(ncplane_set_widget(ncpp->plot.ncp, ncpp, (void(*)(void*))nc##X##plot_destroy)){ \
+    return NULL; \
+  } \
+  ncplot_options zeroed = {0}; \
   if(!opts){ \
     opts = &zeroed; \
   } \
   if(opts->flags >= (NCPLOT_OPTION_PRINTSAMPLE << 1u)){ \
-    logwarn("Provided unsupported flags %016jx\n", (uintmax_t)opts->flags); \
+    logwarn("provided unsupported flags %016" PRIx64, opts->flags); \
   } \
   /* if miny == maxy (enabling domain detection), they both must be equal to 0 */ \
   if(miny == maxy && miny){ \
-    ncplane_destroy(n); \
-    return false; \
+    return NULL; \
   } \
   if(opts->rangex < 0){ \
-    logerror("Supplied negative independent range %d\n", opts->rangex); \
-    ncplane_destroy(n); \
-    return false; \
+    logerror("error: supplied negative independent range %d", opts->rangex); \
+    return NULL; \
   } \
   if(maxy < miny){ \
-    ncplane_destroy(n); \
-    return false; \
+    logerror("error: supplied maxy < miny"); \
+    return NULL; \
   } \
   /* DETECTMAXONLY can't be used without domain detection */ \
   if(opts->flags & NCPLOT_OPTION_DETECTMAXONLY && (miny != maxy)){ \
-    logerror("Supplied DETECTMAXONLY without domain detection"); \
-    ncplane_destroy(n); \
-    return false; \
+    logerror("supplied DETECTMAXONLY without domain detection"); \
+    return NULL; \
   } \
   const notcurses* notc = ncplane_notcurses(n); \
   ncblitter_e blitfxn = opts ? opts->gridtype : NCBLIT_DEFAULT; \
@@ -256,22 +468,20 @@ create_##T(nc##X##plot* ncpp, ncplane* n, const ncplot_options* opts, const T mi
   bool degrade_blitter = !(opts && (opts->flags & NCPLOT_OPTION_NODEGRADE)); \
   const struct blitset* bset = lookup_blitset(&notc->tcache, blitfxn, degrade_blitter); \
   if(bset == NULL){ \
-    ncplane_destroy(n); \
-    return false; \
+    return NULL; \
   } \
-  int sdimy, sdimx; \
+  unsigned sdimy, sdimx; \
   ncplane_dim_yx(n, &sdimy, &sdimx); \
   if(sdimx <= 0){ \
-    ncplane_destroy(n); \
-    return false; \
+    return NULL; \
   } \
-  int dimx = sdimx; \
+  unsigned dimx = sdimx; \
   ncpp->plot.title = strdup(opts->title ? opts->title : ""); \
   ncpp->plot.rangex = opts->rangex; \
   /* if we're sizing the plot based off the plane dimensions, scale it by the \
      plot geometry's width for all calculations */ \
-  const int scaleddim = dimx * bset->width; \
-  const int scaledprefixlen = PREFIXCOLUMNS * bset->width; \
+  const unsigned scaleddim = dimx * (bset->geom == NCBLIT_PIXEL ? ncplane_pile_const(n)->cellpxx : bset->width); \
+  const unsigned scaledprefixlen = NCPREFIXCOLUMNS * (bset->geom == NCBLIT_PIXEL ? ncplane_pile_const(n)->cellpxx : bset->width); \
   if((ncpp->plot.slotcount = ncpp->plot.rangex) == 0){ \
     ncpp->plot.slotcount = scaleddim; \
   } \
@@ -289,11 +499,9 @@ create_##T(nc##X##plot* ncpp, ncplane* n, const ncplot_options* opts, const T mi
   size_t slotsize = sizeof(*ncpp->slots) * ncpp->plot.slotcount; \
   ncpp->slots = malloc(slotsize); \
   if(ncpp->slots == NULL){ \
-    ncplane_destroy(n); \
-    return false; \
+    return NULL; \
   } \
   memset(ncpp->slots, 0, slotsize); \
-  ncpp->plot.ncp = n; \
   ncpp->plot.maxchannels = opts->maxchannels; \
   ncpp->plot.minchannels = opts->minchannels; \
   ncpp->plot.bset = bset; \
@@ -305,12 +513,21 @@ create_##T(nc##X##plot* ncpp, ncplane* n, const ncplot_options* opts, const T mi
   ncpp->plot.printsample = opts->flags & NCPLOT_OPTION_PRINTSAMPLE; \
   if( (ncpp->plot.detectdomain = (miny == maxy)) ){ \
     ncpp->maxy = trueminy; \
-    ncpp->miny = truemaxy; \
+    if(!ncpp->plot.detectonlymax){ \
+      ncpp->miny = truemaxy; \
+    } \
   } \
   ncpp->plot.slotstart = 0; \
   ncpp->plot.slotx = 0; \
+  ncpp->plot.chancount = 0; \
+  ncpp->plot.channels = NULL; \
+  if(bset->geom == NCBLIT_PIXEL){ \
+    if(create_pixelp(&ncpp->plot, n)){ \
+      return NULL; \
+    } \
+  } \
   redraw_plot_##T(ncpp); \
-  return true; \
+  return bset; \
 } \
 /* if x is less than the window, return -1, as the sample will be thrown away. \
    if the x is within the current window, find the proper slot and update it. \
@@ -318,9 +535,7 @@ create_##T(nc##X##plot* ncpp, ncplane* n, const ncplot_options* opts, const T mi
    reset them, and write the new sample anywhere. otherwise, write it to the \
    proper slot based on the current newest slot. */ \
 int window_slide_##T(nc##X##plot* ncp, int64_t x){ \
-  if(x < ncp->plot.slotx - (ncp->plot.slotcount - 1)){ /* x is behind window, won't be counted */ \
-    return -1; \
-  }else if(x <= ncp->plot.slotx){ /* x is within window, do nothing */ \
+  if(x <= ncp->plot.slotx){ /* x is within window, do nothing */ \
     return 0; \
   } /* x is newest; we might be keeping some, might not */ \
   int64_t xdiff = x - ncp->plot.slotx; /* the raw amount we're advancing */ \
@@ -355,7 +570,13 @@ static void update_sample_##T(nc##X##plot* ncp, int64_t x, T y, bool reset); \
    x window, the x window is advanced to include x, and values passing beyond \
    the window are lost. The first call will place the initial window. The plot \
    will be redrawn, but notcurses_render() is not called. */ \
-int add_sample_##T(nc##X##plot* ncpp, uint64_t x, T y){ \
+int add_sample_##T(nc##X##plot* ncpp, int64_t x, T y){ \
+  if(x < ncpp->plot.slotx - (ncpp->plot.slotcount - 1)){ /* x is behind window, won't be counted */ \
+    return -1; \
+  } \
+  if(y == 0 && x <= ncpp->plot.slotx){ \
+    return 0; /* no need to redraw plot; nothing changed */ \
+  } \
   if(window_slide_##T(ncpp, x)){ \
     return -1; \
   } \
@@ -381,7 +602,11 @@ CREATE(double, d)
 static void
 ncplot_destroy(ncplot* n){
   free(n->title);
-  ncplane_destroy(n->ncp);
+  if(ncplane_set_widget(n->ncp, NULL, NULL) == 0){
+    ncplane_destroy(n->ncp);
+  }
+  ncplane_destroy(n->pixelp);
+  free(n->channels);
 }
 
 /* if we're doing domain detection, update the domain to reflect the value we
@@ -457,8 +682,9 @@ ncuplot* ncuplot_create(ncplane* n, const ncplot_options* opts, uint64_t miny, u
     return NULL;
   }
   memset(ret, 0, sizeof(*ret));
-  if(!create_uint64_t(ret, n, opts, miny, maxy, 0, UINT64_MAX)){
-    free(ret);
+  const struct blitset* bset = create_uint64_t(ret, n, opts, miny, maxy, 0, UINT64_MAX);
+  if(bset == NULL){ // create_uint64_t() destroys n on error
+    ncuplot_destroy(ret);
     return NULL;
   }
   return ret;
@@ -499,8 +725,9 @@ ncdplot* ncdplot_create(ncplane* n, const ncplot_options* opts, double miny, dou
     return NULL;
   }
   memset(ret, 0, sizeof(*ret));
-  if(!create_double(ret, n, opts, miny, maxy, -DBL_MAX, DBL_MAX)){
-    free(ret);
+  const struct blitset* bset = create_double(ret, n, opts, miny, maxy, -DBL_MAX, DBL_MAX);
+  if(bset == NULL){ // create_double() destroys n on error
+    ncdplot_destroy(ret);
     return NULL;
   }
   return ret;
