@@ -1,12 +1,35 @@
 #include "internal.h"
 
+static void
+ncreader_destroy_internal(ncreader* n){
+  if(n){
+    if(n->manage_cursor){
+      notcurses_cursor_disable(ncplane_notcurses(n->ncp));
+    }
+    if(ncplane_set_widget(n->ncp, NULL, NULL) == 0){
+      ncplane_destroy(n->ncp);
+    }
+    ncplane_destroy(n->textarea);
+    free(n);
+  }
+}
+
+void ncreader_destroy(ncreader* n, char** contents){
+  if(n){
+    if(contents){
+      *contents = ncreader_contents(n);
+    }
+    ncreader_destroy_internal(n);
+  }
+}
+
 ncreader* ncreader_create(ncplane* n, const ncreader_options* opts){
-  ncreader_options zeroed = {};
+  ncreader_options zeroed = {0};
   if(!opts){
     opts = &zeroed;
   }
   if(opts->flags > NCREADER_OPTION_CURSOR){
-    logwarn("Provided unsupported flags %016jx\n", (uintmax_t)opts->flags);
+    logwarn("provided unsupported flags %016" PRIx64, opts->flags);
   }
   ncreader* nr = malloc(sizeof(*nr));
   if(nr == NULL){
@@ -37,6 +60,12 @@ ncreader* ncreader_create(ncplane* n, const ncreader_options* opts){
   nr->manage_cursor = opts->flags & NCREADER_OPTION_CURSOR;
   ncplane_set_channels(nr->ncp, opts->tchannels);
   ncplane_set_styles(nr->ncp, opts->tattrword);
+  if(ncplane_set_widget(n, nr, (void(*)(void*))ncreader_destroy_internal)){
+    ncplane_destroy(nr->textarea);
+    ncplane_destroy(nr->ncp);
+    free(nr);
+    return NULL;
+  }
   return nr;
 }
 
@@ -61,10 +90,10 @@ ncreader_redraw(ncreader* n){
   assert(n->xproject >= 0);
   assert(n->textarea->lenx >= n->ncp->lenx);
   assert(n->textarea->leny >= n->ncp->leny);
-  for(int y = 0 ; y < n->ncp->leny ; ++y){
-    const int texty = y;
-    for(int x = 0 ; x < n->ncp->lenx ; ++x){
-      const int textx = x + n->xproject;
+  for(unsigned y = 0 ; y < n->ncp->leny ; ++y){
+    const unsigned texty = y;
+    for(unsigned x = 0 ; x < n->ncp->lenx ; ++x){
+      const unsigned textx = x + n->xproject;
       const nccell* src = &n->textarea->fb[nfbcellidx(n->textarea, texty, textx)];
       nccell* dst = &n->ncp->fb[nfbcellidx(n->ncp, y, x)];
 //fprintf(stderr, "projecting %d/%d [%s] to %d/%d [%s]\n", texty, textx, cell_extended_gcluster(n->textarea, src), y, x, cell_extended_gcluster(n->ncp, dst));
@@ -121,9 +150,9 @@ int ncreader_move_left(ncreader* n){
 // row. if on the right side of the viewarea, but not the right side of the
 // textarea, pans right. returns 0 if a move was made.
 int ncreader_move_right(ncreader* n){
-  int viewx = n->ncp->x;
-  int textx = n->textarea->x;
-  int y = n->ncp->y;
+  unsigned textx = n->textarea->x;
+  unsigned y = n->ncp->y;
+  unsigned viewx = n->ncp->x;
 //fprintf(stderr, "moving right: tcurs: %dx%d vcurs: %dx%d xproj: %d\n", y, textx, y, viewx, n->xproject);
   if(textx >= n->textarea->lenx - 1){
     // are we on the last column of the textarea? if so, we must also be on
@@ -171,7 +200,7 @@ int ncreader_move_up(ncreader* n){
 // try to move down. does not move past the bottom of the textarea.
 // returns 0 if a move was made.
 int ncreader_move_down(ncreader* n){
-  int y = n->ncp->y;
+  unsigned y = n->ncp->y;
   if(y >= n->textarea->leny - 1){
     // are we on the last row of the textarea? if so, we can't move.
     return -1;
@@ -185,9 +214,9 @@ int ncreader_move_down(ncreader* n){
 
 // only writing can enlarge the textarea. movement can pan, but not enlarge.
 int ncreader_write_egc(ncreader* n, const char* egc){
-  const int cols = ncstrwidth(egc);
+  const int cols = ncstrwidth(egc, NULL, NULL);
   if(cols < 0){
-    logerror("Fed illegal UTF-8 [%s]\n", egc);
+    logerror("fed illegal UTF-8 [%s]", egc);
     return -1;
   }
   if(n->textarea->x >= n->textarea->lenx - cols){
@@ -340,12 +369,15 @@ ncreader_alt_input(ncreader* n, const ncinput* ni){
 //  * anything with Ctrl, except 'U' (which clears all input)
 //  * anything synthesized, save arrow keys and backspace
 bool ncreader_offer_input(ncreader* n, const ncinput* ni){
-  if(ni->ctrl && !n->no_cmd_keys){
+  if(ni->evtype == NCTYPE_RELEASE){
+    return false;
+  }
+  if(ncinput_ctrl_p(ni) && !n->no_cmd_keys){
     return ncreader_ctrl_input(n, ni);
-  }else if(ni->alt && !n->no_cmd_keys){
+  }else if(ncinput_alt_p(ni) && !n->no_cmd_keys){
     return ncreader_alt_input(n, ni);
   }
-  if(ni->alt || ni->ctrl){ // pass on all alts/ctrls if no_cmd_keys is set
+  if(ncinput_alt_p(ni) || ncinput_ctrl_p(ni)){ // pass on all alts/ctrls if no_cmd_keys is set
     return false;
   }
   if(ni->id == NCKEY_BACKSPACE){
@@ -365,7 +397,7 @@ bool ncreader_offer_input(ncreader* n, const ncinput* ni){
   }else if(ni->id == NCKEY_DOWN){
     ncreader_move_down(n);
     return true;
-  }else if(nckey_supppuab_p(ni->id)){
+  }else if(nckey_synthesized_p(ni->id)){
     return false;
   }
   // FIXME need to collect full EGCs
@@ -378,19 +410,5 @@ bool ncreader_offer_input(ncreader* n, const ncinput* ni){
 }
 
 char* ncreader_contents(const ncreader* n){
-  return ncplane_contents(n->ncp, 0, 0, -1, -1);
-}
-
-void ncreader_destroy(ncreader* n, char** contents){
-  if(n){
-    if(contents){
-      *contents = ncreader_contents(n);
-    }
-    if(n->manage_cursor){
-      notcurses_cursor_disable(ncplane_notcurses(n->ncp));
-    }
-    ncplane_destroy(n->textarea);
-    ncplane_destroy(n->ncp);
-    free(n);
-  }
+  return ncplane_contents(n->ncp, 0, 0, 0, 0);
 }
