@@ -5,7 +5,6 @@ using namespace ncpp;
 
 notcurses_options NotCurses::default_notcurses_options = {
 	/* termtype */                 nullptr,
-	/* renderfp */                 nullptr,
 	/* loglevel */                 NCLogLevel::Silent,
 	/* margin_t */                 0,
 	/* margin_r */                 0,
@@ -36,7 +35,7 @@ NotCurses::NotCurses (const notcurses_options &nc_opts, FILE *fp)
 
 	nc = notcurses_init (&nc_opts, fp);
 	if (nc == nullptr)
-		throw new init_error ("notcurses failed to initialize");
+		throw init_error ("notcurses failed to initialize");
 	if (_instance == nullptr)
 		_instance = this;
 }
