@@ -1,5 +1,6 @@
 #include <array>
 #include <memory>
+#include <cinttypes>
 #include <cstring>
 #include <cstdlib>
 #include <clocale>
@@ -27,7 +28,7 @@ void usage(std::ostream& o, const char* name, int exitcode){
   o << " -k: use direct mode (cannot be used with -L or -d)\n";
   o << " -L: loop frames\n";
   o << " -t seconds: delay t seconds after each file\n";
-  o << " -l loglevel: integer between 0 and 7, goes to stderr'\n";
+  o << " -l loglevel: integer between 0 and 7, goes to stderr\n";
   o << " -s scaling: one of 'none', 'hires', 'scale', 'scalehi', or 'stretch'\n";
   o << " -b blitter: one of 'ascii', 'half', 'quad', 'sex', 'braille', or 'pixel'\n";
   o << " -m margins: margin, or 4 comma-separated margins\n";
@@ -47,9 +48,10 @@ struct marshal {
 auto perframe(struct ncvisual* ncv, struct ncvisual_options* vopts,
               const struct timespec* abstime, void* vmarshal) -> int {
   struct marshal* marsh = static_cast<struct marshal*>(vmarshal);
-  NotCurses &nc = NotCurses::get_instance ();
+  NotCurses &nc = NotCurses::get_instance();
   auto start = static_cast<struct timespec*>(ncplane_userptr(vopts->n));
   if(!start){
+    // FIXME how do we get this free()d at the end?
     start = static_cast<struct timespec*>(malloc(sizeof(struct timespec)));
     clock_gettime(CLOCK_MONOTONIC, start);
     ncplane_set_userptr(vopts->n, start);
@@ -87,7 +89,7 @@ auto perframe(struct ncvisual* ncv, struct ncvisual_options* vopts,
   if(!nc.render()){
     return -1;
   }
-  int dimx, dimy, oldx, oldy;
+  unsigned dimx, dimy, oldx, oldy;
   nc.get_term_dim(&dimy, &dimx);
   ncplane_dim_yx(vopts->n, &oldy, &oldx);
   uint64_t absnow = timespec_to_ns(abstime);
@@ -101,31 +103,37 @@ auto perframe(struct ncvisual* ncv, struct ncvisual_options* vopts,
       ns_to_timespec(absnow - nsnow, &interval);
       keyp = nc.get(&interval, &ni);
     }else{
-      keyp = nc.get();
+      keyp = nc.get(false, &ni);
     }
-    if(keyp == (uint32_t)-1){
+    if(keyp == 0){
       break;
     }
-    if(keyp == ' '){
-      if((keyp = nc.get(true)) == (uint32_t)-1){
-        ncplane_destroy(subp);
-        return -1;
-      }
-    }
-    // if we just hit a non-space character to unpause, interpret it
-    if(keyp == ' '){ // space for unpause
+    // we don't care about key release events, especially the enter
+    // release that starts so many interactive programs under Kitty
+    if(ni.evtype == EvType::Release){
       continue;
     }
+    if(keyp == ' '){
+      do{
+        if((keyp = nc.get(true, &ni)) == (uint32_t)-1){
+          ncplane_destroy(subp);
+          return -1;
+        }
+      }while(ni.id != 'q' && (ni.evtype == EvType::Release || ni.id != ' '));
+    }
+    // if we just hit a non-space character to unpause, ignore it
     if(keyp == NCKey::Resize){
       return 0;
-    }else if(keyp == 'L' && ni.ctrl && !ni.alt){
+    }else if(keyp == ' '){ // space for unpause
+      continue;
+    }else if(keyp == 'L' && ncinput_ctrl_p(&ni) && !ncinput_alt_p(&ni)){
       nc.refresh(nullptr, nullptr);
       continue;
-    }else if(keyp >= '0' && keyp <= '6' && !ni.alt && !ni.ctrl){
+    }else if(keyp >= '0' && keyp <= '6' && !ncinput_alt_p(&ni) && !ncinput_ctrl_p(&ni)){
       marsh->blitter = static_cast<ncblitter_e>(keyp - '0');
       vopts->blitter = marsh->blitter;
       continue;
-    }else if(keyp >= '7' && keyp <= '9' && !ni.alt && !ni.ctrl){
+    }else if(keyp >= '7' && keyp <= '9' && !ncinput_alt_p(&ni) && !ncinput_ctrl_p(&ni)){
       continue; // don't error out
     }else if(keyp == NCKey::Up){
       // FIXME move backwards significantly
@@ -139,6 +147,8 @@ auto perframe(struct ncvisual* ncv, struct ncvisual_options* vopts,
     }else if(keyp == NCKey::Left){
       // FIXME move backwards
       continue;
+    }else if(keyp != 'q'){
+      continue;
     }
     ncplane_destroy(subp);
     return 1;
@@ -151,7 +161,7 @@ auto perframe(struct ncvisual* ncv, struct ncvisual_options* vopts,
 auto handle_opts(int argc, char** argv, notcurses_options& opts, bool* quiet,
                  float* timescale, ncscale_e* scalemode, ncblitter_e* blitter,
                  float* displaytime, bool* loop, bool* noninterp,
-                 uint32_t* transcolor)
+                 uint32_t* transcolor, bool* climode)
                  -> int {
   *timescale = 1.0;
   *scalemode = NCSCALE_STRETCH;
@@ -199,13 +209,17 @@ auto handle_opts(int argc, char** argv, notcurses_options& opts, bool* quiet,
         break;
       case 'b':
         if(notcurses_lex_blitter(optarg, blitter)){
-          std::cerr << "Invalid blitter specification (got "
-                    << optarg << ")" << std::endl;
+          std::cerr << "Invalid blitter specification (got " << optarg << ")" << std::endl;
           usage(std::cerr, argv[0], EXIT_FAILURE);
         }
         break;
       case 'k':{ // actually engages direct mode
-        opts.flags |= NCOPTION_NO_ALTERNATE_SCREEN;
+        opts.flags |= NCOPTION_NO_ALTERNATE_SCREEN
+                      | NCOPTION_PRESERVE_CURSOR
+                      | NCOPTION_NO_CLEAR_BITMAPS;
+        *displaytime = 0;
+        *quiet = true;
+        *climode = true;
         if(*loop || *timescale != 1.0){
           std::cerr << "-k cannot be used with -L or -d" << std::endl;
           usage(std::cerr, argv[0], EXIT_FAILURE);
@@ -277,83 +291,23 @@ auto handle_opts(int argc, char** argv, notcurses_options& opts, bool* quiet,
   if(argv[optind] == nullptr){
     usage(std::cerr, argv[0], EXIT_FAILURE);
   }
+  if(*blitter == NCBLIT_DEFAULT){
+    *blitter = NCBLIT_PIXEL;
+  }
   return optind;
 }
 
-int perframe_direct(struct ncvisual* ncv, struct ncvisual_options* vopts,
-                    const struct timespec* abstime, void* vmarshal){
-  // FIXME probably want to reset cursor here?
-  (void)ncv;
-  (void)vopts;
-  (void)abstime;
-  (void)vmarshal;
-  return 0;
-}
-
-// argc/argv ought already be reduced to only the media arguments
-int direct_mode_player(int argc, char** argv, ncscale_e scalemode,
-                       ncblitter_e blitter, int lmargin,
-                       bool noninterp, unsigned transcolor,
-                       ncloglevel_e loglevel){
-  uint64_t flags = loglevel > NCLOGLEVEL_ERROR ?
-                    loglevel > NCLOGLEVEL_WARNING ?
-                     NCDIRECT_OPTION_VERY_VERBOSE : NCDIRECT_OPTION_VERBOSE : 0;
-  Direct dm{nullptr, nullptr, flags};
-  if(!dm.canopen_images()){
-    std::cerr << "Notcurses was compiled without multimedia support\n";
-    return -1;
-  }
-  bool failed = false;
-  if(blitter == NCBLIT_PIXEL){
-    if(dm.check_pixel_support() <= 0){
-      blitter = NCBLIT_DEFAULT;
-    }
-  }
-  dm.cursor_disable();
-  for(auto i = 0 ; i < argc ; ++i){
-    // FIXME need to free faken
-    // FIXME we want to honor the different left and right margins, but that
-    // would require raster_image() knowing how far over we were starting for
-    // multiline cellular blittings...
-    ncpp::NCAlign a;
-    if(blitter == NCBLIT_PIXEL){
-      printf("%*.*s", lmargin, lmargin, "");
-      a = NCAlign::Left;
-    }else{
-      a = NCAlign::Center;
-    }
-    struct ncvisual_options vopts{};
-    vopts.blitter = blitter;
-    vopts.scaling = scalemode;
-    vopts.x = static_cast<int>(a);
-    vopts.flags = NCVISUAL_OPTION_HORALIGNED;
-    if(noninterp){
-      vopts.flags |= NCVISUAL_OPTION_NOINTERPOLATE;
-    }
-    if(transcolor){
-      vopts.flags |= NCVISUAL_OPTION_ADDALPHA;
-    }
-    vopts.transcolor = transcolor & 0xffffffull;
-    if(dm.streamfile(argv[i], perframe_direct, &vopts, NULL)){
-      failed = true;
-    }
-    int y, x;
-    dm.get_cursor_yx(&y, &x);
-    if(x){
-      std::cout << std::endl;
-    }
-  }
-  dm.cursor_enable();
-  return failed ? -1 : 0;
-}
-
 int rendered_mode_player_inner(NotCurses& nc, int argc, char** argv,
                                ncscale_e scalemode, ncblitter_e blitter,
                                bool quiet, bool loop,
                                double timescale, double displaytime,
-                               bool noninterp, uint32_t transcolor){
-  int dimy, dimx;
+                               bool noninterp, uint32_t transcolor,
+                               bool climode){
+  unsigned dimy, dimx;
   std::unique_ptr<Plane> stdn(nc.get_stdplane(&dimy, &dimx));
+  if(climode){
+    stdn->set_scrolling(true);
+  }
   uint64_t transchan = 0;
   ncchannels_set_fg_alpha(&transchan, NCALPHA_TRANSPARENT);
   ncchannels_set_bg_alpha(&transchan, NCALPHA_TRANSPARENT);
@@ -362,17 +316,17 @@ int rendered_mode_player_inner(NotCurses& nc, int argc, char** argv,
   nopts.name = "play";
   nopts.resizecb = ncplane_resize_marginalized;
   nopts.flags = NCPLANE_OPTION_MARGINALIZED;
+  ncplane* n = nullptr;
+  ncplane* clin = nullptr;
   for(auto i = 0 ; i < argc ; ++i){
     std::unique_ptr<Visual> ncv;
     ncv = std::make_unique<Visual>(argv[i]);
-    auto n = ncplane_create(*stdn, &nopts);
-    if(!n){
+    if((n = ncplane_create(*stdn, &nopts)) == nullptr){
       return -1;
     }
     ncplane_move_bottom(n);
     struct ncvisual_options vopts{};
     int r;
-    vopts.flags |= NCVISUAL_OPTION_HORALIGNED | NCVISUAL_OPTION_VERALIGNED;
     if(noninterp){
       vopts.flags |= NCVISUAL_OPTION_NOINTERPOLATE;
     }
@@ -380,11 +334,30 @@ int rendered_mode_player_inner(NotCurses& nc, int argc, char** argv,
       vopts.flags |= NCVISUAL_OPTION_ADDALPHA;
     }
     vopts.transcolor = transcolor & 0xffffffull;
-    vopts.y = NCALIGN_CENTER;
-    vopts.x = NCALIGN_CENTER;
     vopts.n = n;
     vopts.scaling = scalemode;
     vopts.blitter = blitter;
+    if(!climode){
+      vopts.flags |= NCVISUAL_OPTION_HORALIGNED | NCVISUAL_OPTION_VERALIGNED;
+      vopts.y = NCALIGN_CENTER;
+      vopts.x = NCALIGN_CENTER;
+    }else{
+      ncvgeom geom;
+      if(ncvisual_geom(nc, *ncv, &vopts, &geom)){
+        return -1;
+      }
+      struct ncplane_options cliopts{};
+      cliopts.y = stdn->cursor_y();
+      cliopts.x = stdn->cursor_x();
+      cliopts.rows = geom.rcelly;
+      cliopts.cols = geom.rcellx;
+      clin = ncplane_create(n, &cliopts);
+      if(!clin){
+        return -1;
+      }
+      vopts.n = clin;
+      ncplane_scrollup_child(*stdn, clin);
+    }
     ncplane_erase(n);
     do{
       struct marshal marsh = {
@@ -399,31 +372,36 @@ int rendered_mode_player_inner(NotCurses& nc, int argc, char** argv,
         vopts.blitter = marsh.blitter;
         if(!loop){
           if(displaytime < 0){
-            stdn->printf(0, NCAlign::Center, "press a key to advance");
+            stdn->printf(0, NCAlign::Center, "press space to advance");
             if(!nc.render()){
-              ncplane_destroy(n);
-              return -1;
+              goto err;
             }
-            uint32_t ie = nc.get(true);
-            if(ie == (uint32_t)-1){
-              return -1;
-            }else if(ie == 'q'){
-              return 0;
-            }else if(ie == 'L'){
-              --i;
-              nc.refresh(nullptr, nullptr);
-            }else if(ie >= '0' && ie <= '6'){
-              blitter = vopts.blitter = static_cast<ncblitter_e>(ie - '0');
-              --i; // rerun same input with the new blitter
-            }else if(ie >= '7' && ie <= '9'){
-              --i; // just absorb the input
-            }else if(ie == NCKey::Resize){
-              --i; // rerun with the new size
-              if(!nc.refresh(&dimy, &dimx)){
-                ncplane_destroy(n);
+            ncinput ni;
+            do{
+              do{
+                nc.get(true, &ni);
+              }while(ni.evtype == EvType::Release);
+              if(ni.id == (uint32_t)-1){
                 return -1;
+              }else if(ni.id == 'q'){
+                return 0;
+              }else if(ni.id == 'L'){
+                nc.refresh(nullptr, nullptr);
+              }else if(ni.id >= '0' && ni.id <= '6'){
+                blitter = vopts.blitter = static_cast<ncblitter_e>(ni.id - '0');
+                --i; // rerun same input with the new blitter
+                break;
+              }else if(ni.id >= '7' && ni.id <= '9'){
+                --i; // just absorb the input
+                break;
+              }else if(ni.id == NCKey::Resize){
+                --i; // rerun with the new size
+                if(!nc.refresh(&dimy, &dimx)){
+                  goto err;
+                }
+                break;
               }
-            }
+            }while(ni.id != ' ');
           }else{
             // FIXME do we still want to honor keybindings when timing out?
             struct timespec ts;
@@ -434,24 +412,30 @@ int rendered_mode_player_inner(NotCurses& nc, int argc, char** argv,
         }else{
           ncv->decode_loop();
         }
+        ncplane_destroy(clin);
       }
     }while(loop && r == 0);
     if(r < 0){ // positive is intentional abort
       std::cerr << "Error while playing " << argv[i] << std::endl;
-      ncplane_destroy(n);
-      return -1;
+      goto err;
     }
     free(ncplane_userptr(n));
     ncplane_destroy(n);
   }
   return 0;
+
+err:
+  free(ncplane_userptr(n));
+  ncplane_destroy(n);
+  return -1;
 }
 
 int rendered_mode_player(int argc, char** argv, ncscale_e scalemode,
                          ncblitter_e blitter, notcurses_options& ncopts,
                          bool quiet, bool loop,
                          double timescale, double displaytime,
-                         bool noninterp, uint32_t transcolor){
+                         bool noninterp, uint32_t transcolor,
+                         bool climode){
   // no -k, we're using full rendered mode (and the alternate screen).
   ncopts.flags |= NCOPTION_INHIBIT_SETLOCALE;
   if(quiet){
@@ -467,7 +451,7 @@ int rendered_mode_player(int argc, char** argv, ncscale_e scalemode,
     }
     r = rendered_mode_player_inner(nc, argc, argv, scalemode, blitter,
                                    quiet, loop, timescale, displaytime,
-                                   noninterp, transcolor);
+                                   noninterp, transcolor, climode);
     if(!nc.stop()){
       return -1;
     }
@@ -494,19 +478,15 @@ auto main(int argc, char** argv) -> int {
   bool quiet = false;
   bool loop = false;
   bool noninterp = false;
+  bool climode = false;
   auto nonopt = handle_opts(argc, argv, ncopts, &quiet, &timescale, &scalemode,
-                            &blitter, &displaytime, &loop, &noninterp, &transcolor);
-  int r;
-  // if -k was provided, we now use direct mode rather than simply not using the
+                            &blitter, &displaytime, &loop, &noninterp, &transcolor,
+                            &climode);
+  // if -k was provided, we use CLI mode rather than simply not using the
   // alternate screen, so that output is inline with the shell.
-  if(ncopts.flags & NCOPTION_NO_ALTERNATE_SCREEN){
-    r = direct_mode_player(argc - nonopt, argv + nonopt, scalemode, blitter,
-                           ncopts.margin_l, noninterp, transcolor, ncopts.loglevel);
-  }else{
-    r = rendered_mode_player(argc - nonopt, argv + nonopt, scalemode, blitter, ncopts,
-                             quiet, loop, timescale, displaytime, noninterp, transcolor);
-  }
-  if(r){
+  if(rendered_mode_player(argc - nonopt, argv + nonopt, scalemode, blitter, ncopts,
+                          quiet, loop, timescale, displaytime, noninterp,
+                          transcolor, climode)){
     return EXIT_FAILURE;
   }
   return EXIT_SUCCESS;
