@@ -19,6 +19,7 @@ int main(void){
   ncplane_set_styles(n, NCSTYLE_BOLD);
   ncplane_putstr(n, "This program is *not* indicative of real scrolling speed.\n");
   ncplane_set_styles(n, NCSTYLE_NONE);
+  unsigned y = ncplane_cursor_y(n);
   while(true){
     struct timespec req = { .tv_sec = 0, .tv_nsec = 1000000, };
     nanosleep(&req, NULL);
@@ -28,10 +29,16 @@ int main(void){
     if(++c == '{'){
       c = 'A';
     }
-    if(notcurses_render(nc)){
-      break;
+    unsigned newy = ncplane_cursor_y(n);
+    if(newy != y){
+      y = newy;
+      notcurses_render(nc);
     }
   }
+  if(notcurses_render(nc)){
+    notcurses_stop(nc);
+    return EXIT_FAILURE;
+  }
   notcurses_stop(nc);
   return EXIT_SUCCESS;
 }
