@@ -1,6 +1,5 @@
 #define DOCTEST_CONFIG_IMPLEMENT
 #include "main.h"
-#include <term.h>
 #include <fcntl.h>
 #include <clocale>
 #include <cstring>
@@ -8,17 +7,21 @@
 #include <iostream>
 #include <climits>
 #include <sys/stat.h>
-#include <filesystem>
 
-const char* datadir = NOTCURSES_SHARE;
-ncloglevel_e cliloglevel = NCLOGLEVEL_SILENT;
+const char* datadir = notcurses_data_dir();
+
+// we define loglevel for any use of logging in internal header files.
+// note that this has no bearing on the library's true inner loglevel!
+ncloglevel_e loglevel;
 
 auto testing_notcurses() -> struct notcurses* {
   notcurses_options nopts{};
   // get loglevel from command line. enabling it by default leads to
   // more confusion than useful information, so leave it off by default.
-  nopts.loglevel = cliloglevel;
-  nopts.flags = NCOPTION_SUPPRESS_BANNERS | NCOPTION_NO_ALTERNATE_SCREEN;
+  nopts.loglevel = loglevel;
+  nopts.flags = NCOPTION_SUPPRESS_BANNERS
+                | NCOPTION_NO_ALTERNATE_SCREEN
+                | NCOPTION_DRAIN_INPUT;
   auto nc = notcurses_init(&nopts, nullptr);
   return nc;
 }
@@ -26,9 +29,10 @@ auto testing_notcurses() -> struct notcurses* {
 template <typename T> using uniqptr = std::unique_ptr<T,free_deleter>;
 
 auto find_data(const char* datum) -> uniqptr<char> {
-  std::filesystem::path p = datadir;
-  p /= datum;
-  uniqptr<char> uptr(strdup(p.c_str()));
+  std::string s = datadir;
+  s += path_separator();
+  s += datum;
+  uniqptr<char> uptr(strdup(s.c_str()));
   return uptr;
 }
 
@@ -43,6 +47,9 @@ auto is_test_tty() -> bool {
 
 static void
 handle_opts(const char** argv){
+  // now that we've spun up one testing framework, switch to _SILENT unless
+  // something else has been provided on the command line.
+  loglevel = NCLOGLEVEL_SILENT;
   bool inarg = false;
   while(*argv){
     if(inarg){
@@ -51,10 +58,34 @@ handle_opts(const char** argv){
     }else if(strcmp(*argv, "-p") == 0){
       inarg = true;
     }else if(strncmp(*argv, "-l", 2) == 0){ // just require -l
-      cliloglevel = NCLOGLEVEL_TRACE;
+      char* eol;
+      long ll = strtol(*argv + 2, &eol, 0);
+      if(ll < NCLOGLEVEL_SILENT || ll > NCLOGLEVEL_TRACE){
+        std::cerr << "illegal loglevel: " << *argv + 2 << std::endl;
+        exit(EXIT_FAILURE);
+      }
+      if(*eol){
+        std::cerr << "illegal loglevel: " << *argv + 2 << std::endl;
+        exit(EXIT_FAILURE);
+      }
+      std::cout << "got loglevel " << ll << std::endl;
+      loglevel = static_cast<ncloglevel_e>(ll);
     }
     ++argv;
   }
+  auto nc = testing_notcurses();
+  if(!nc){
+    std::cerr << "Couldn't create notcurses testing framework" << std::endl;
+    exit(EXIT_FAILURE);
+  }
+  unsigned dimy, dimx;
+  notcurses_stddim_yx(nc, &dimy, &dimx);
+  std::cout << "Detected cell geometry: " << dimx << 'x' << dimy << std::endl;
+  notcurses_stop(nc);
+  if(dimx < 50 || dimy < 24){ // minimum assumed geometry
+    std::cerr << "Terminal was too small for tests (minimum 50x24)" << std::endl;
+    exit(EXIT_FAILURE);
+  }
 }
 
 // check that the (provided or default) data directory exists, and has at
@@ -86,6 +117,7 @@ reset_terminal(){
       tios.c_lflag |= ISIG | ICANON | ECHO;
       tcsetattr(fd, TCSADRAIN, &tios);
     }
+    printf(KKEYBOARD_POP);
     char* str = tigetstr("sgr0");
     if(str != (char*)-1){
       printf("%s", str);
@@ -127,25 +159,15 @@ auto lang_and_term() -> void {
   }else{
     std::cout << "Running with LANG=" << lang << std::endl;
   }
+#ifndef __MINGW32__
   const char* term = getenv("TERM");
   // ubuntu's buildd sets TERM=unknown, fuck it, handle this atrocity
   if(term == nullptr || strcmp(term, "unknown") == 0){
     std::cerr << "TERM wasn't defined, exiting with success" << std::endl;
     exit(EXIT_SUCCESS);
   }
-  auto nc = testing_notcurses();
-  if(!nc){
-    exit(EXIT_FAILURE);
-  }
-  int dimy, dimx;
-  notcurses_stddim_yx(nc, &dimy, &dimx);
-  std::cout << "Running with TERM=" << term << " (" << dimx << 'x'
-            << dimy << ")" << std::endl;
-  notcurses_stop(nc);
-  if(dimx < 80 || dimy < 24){ // minimum assumed geometry
-    std::cerr << "Terminal was too small for tests (minimum 80x24)" << std::endl;
-    exit(EXIT_FAILURE);
-  }
+  std::cout << "Running with TERM=" << term << std::endl;
+#endif
 }
 
 auto main(int argc, const char **argv) -> int {
@@ -163,7 +185,9 @@ auto main(int argc, const char **argv) -> int {
   handle_opts(argv);
   int res = context.run(); // run
   reset_terminal();
-  check_data_dir();
+  if(res){
+    check_data_dir();
+  }
   if(context.shouldExit()){ // important - query flags (and --exit) rely on the user doing this
     return res;             // propagate the result of the tests
   }
