@@ -5,11 +5,12 @@
 #include <unistd.h>
 #include "version.h"
 #include "builddef.h"
+#include <signal.h>
 #include <doctest/doctest.h>
 #include <notcurses/notcurses.h>
 #include <ncpp/NotCurses.hh>
 #include <ncpp/_exceptions.hh>
-#include "internal.h"
+#include "lib/internal.h"
 
 struct free_deleter{
   template <typename T>
