@@ -1,6 +1,6 @@
 #include "main.h"
 
-TEST_CASE("Palette256") {
+TEST_CASE("Palette") {
   auto nc_ = testing_notcurses();
   if(!nc_){
     return;
@@ -17,7 +17,7 @@ TEST_CASE("Palette256") {
   SUBCASE("SetIndexZero") {
     ncpalette* p = ncpalette_new(nc_);
     REQUIRE(nullptr != p);
-    ncpalette_set_rgb8(p, 0, 0x80, 0x90, 0xa0);
+    CHECK(0 == ncpalette_set_rgb8(p, 0, 0x80, 0x90, 0xa0));
     unsigned r, g, b;
     ncpalette_get_rgb8(p, 0, &r, &g, &b);
     CHECK(r == 0x80);
@@ -29,7 +29,7 @@ TEST_CASE("Palette256") {
   SUBCASE("SetIndex255") {
     ncpalette* p = ncpalette_new(nc_);
     REQUIRE(nullptr != p);
-    ncpalette_set_rgb8(p, 255, 0xa0, 0x70, 0x50);
+    CHECK(0 == ncpalette_set_rgb8(p, 255, 0xa0, 0x70, 0x50));
     unsigned r, g, b;
     ncpalette_get_rgb8(p, 255, &r, &g, &b);
     CHECK(r == 0xa0);
@@ -40,9 +40,9 @@ TEST_CASE("Palette256") {
 
   // when we set a palette index, it ought change us from using default
   SUBCASE("FAttributes") {
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(nccell_fg_default_p(&c));
-    nccell_set_fg_alpha(&c, NCALPHA_TRANSPARENT);
+    CHECK(0 == nccell_set_fg_alpha(&c, NCALPHA_TRANSPARENT));
     CHECK(0 == nccell_set_fg_palindex(&c, 0x20));
     CHECK(!nccell_fg_default_p(&c));
     CHECK(nccell_fg_palindex_p(&c));
@@ -51,9 +51,9 @@ TEST_CASE("Palette256") {
   }
 
   SUBCASE("BAttributes") {
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(nccell_bg_default_p(&c));
-    nccell_set_bg_alpha(&c, NCALPHA_TRANSPARENT);
+    CHECK(0 == nccell_set_bg_alpha(&c, NCALPHA_TRANSPARENT));
     CHECK(0 == nccell_set_bg_palindex(&c, 0x20));
     CHECK(!nccell_bg_default_p(&c));
     CHECK(nccell_bg_palindex_p(&c));
@@ -63,13 +63,13 @@ TEST_CASE("Palette256") {
 
   // write it to an ncplane, and verify attributes via reflection
   SUBCASE("PutCAttrs") {
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(1 == nccell_load_char(n_, &c, 'X'));
     CHECK(0 == nccell_set_fg_palindex(&c, 0x20));
     CHECK(0 == nccell_set_bg_palindex(&c, 0x40));
     CHECK(1 == ncplane_putc_yx(n_, 0, 0, &c));
     nccell_release(n_, &c);
-    nccell r = CELL_TRIVIAL_INITIALIZER;
+    nccell r = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(0 < ncplane_at_yx_cell(n_, 0, 0, &r));
     CHECK(nccell_fg_palindex_p(&r));
     CHECK(nccell_bg_palindex_p(&r));
@@ -81,17 +81,19 @@ TEST_CASE("Palette256") {
   }
 
   SUBCASE("RenderCAttrs") {
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     nccell_load_char(n_, &c, 'X');
     CHECK(0 == nccell_set_fg_palindex(&c, 0x20));
     CHECK(0 == nccell_set_bg_palindex(&c, 0x40));
-    CHECK(0 == ncplane_set_fg_palindex(n_, 0x20));
-    CHECK(0 == ncplane_set_bg_palindex(n_, 0x40));
+    CHECK(0 == nccell_set_fg_alpha(&c, NCALPHA_OPAQUE));
+    CHECK(0 == nccell_set_bg_alpha(&c, NCALPHA_OPAQUE));
     CHECK(0 < ncplane_putc_yx(n_, 0, 0, &c));
     nccell_release(n_, &c);
     CHECK(0 == notcurses_render(nc_));
-    nccell r = CELL_TRIVIAL_INITIALIZER;
-    CHECK(nullptr != notcurses_at_yx(nc_, 0, 0, &r.stylemask, &r.channels));
+    nccell r = NCCELL_TRIVIAL_INITIALIZER;
+    auto egc = notcurses_at_yx(nc_, 0, 0, &r.stylemask, &r.channels);
+    CHECK(nullptr != egc);
+    free(egc);
     CHECK(nccell_fg_palindex_p(&r));
     CHECK(nccell_bg_palindex_p(&r));
     CHECK(NCALPHA_OPAQUE == nccell_fg_alpha(&r));
