@@ -7,7 +7,7 @@ TEST_CASE("Readers") {
   if(!nc_){
     return;
   }
-  int dimx, dimy;
+  unsigned dimx, dimy;
   struct ncplane* n_ = notcurses_stddim_yx(nc_, &dimy, &dimx);
   REQUIRE(n_);
   REQUIRE(0 == ncplane_cursor_move_yx(n_, 0, 0));
@@ -27,14 +27,14 @@ TEST_CASE("Readers") {
     };
     auto ncp = ncplane_create(notcurses_stdplane(nc_), &nopts);
     uint64_t echannels = NCCHANNELS_INITIALIZER(0xff, 0x44, 0xff, 0, 0, 0);
-    ncplane_set_base(ncp, notcurses_canutf8(nc_) ? strdup("▒") : strdup("x"),
-                     0, echannels);
+    ncplane_set_base(ncp, notcurses_canutf8(nc_) ? "\u2592" : "x", 0, echannels);
     auto nr = ncreader_create(ncp, &opts);
     REQUIRE(nullptr != nr);
     CHECK(0 == notcurses_render(nc_));
     char* contents = nullptr;
     ncreader_destroy(nr, &contents);
     REQUIRE(contents);
+    free(contents);
     CHECK(0 == notcurses_render(nc_));
   }
 
