@@ -12,6 +12,9 @@ OLDVERSION="$1"
 VERSION="$2"
 QUIP="$3"
 
+# update the FFI exports
+python tools/generate_ffi.py .
+
 # finalize the date on the most recent version, add any last-minute notes?
 vi NEWS.md
 
@@ -35,8 +38,9 @@ done
 sed -i -e "s/v$OLDVERSION/v$VERSION/g" doc/man/index.html
 sed -i -e "s/version=\"$OLDVERSION\"/version=\"$VERSION\"/" cffi/setup.py
 sed -i -e "s/version=\"$OLDVERSION\"/version=\"$VERSION\"/" python/setup.py
-sed -i -e "s/^version = \"$OLDVERSION\"$/version = \"$VERSION\"/" rust/Cargo.toml
-sed -i -e "s/atleast_version(\"$OLDVERSION\")/atleast_version(\"$VERSION\")/" rust/build/build.rs
+sed -i -e "s/<version>$OLDVERSION</<version>$VERSION</" tools/nuspec
+sed -i -e "s/Program version = $OLDVERSION/Program version = $VERSION/" tools/notcurses-installer.ifp
+sed -i -e "s/File = \(.*\)notcurses-$OLDVERSION/File = \1notcurses-$VERSION/" tools/notcurses-installer.ifp
 
 BUILDDIR="build-$VERSION"
 
@@ -61,7 +65,7 @@ git pull
 TARBALL=v$VERSION.tar.gz
 wget https://github.com/dankamongmen/notcurses/archive/$TARBALL
 gpg --sign --armor --detach-sign $TARBALL
-rm v$VERSION.tar.gz
+rm $TARBALL
 
 echo "Cut $VERSION, signed to $TARBALL.asc"
 echo "Now uploadling the sig to https://github.com/dankamongmen/notcurses/releases"
@@ -74,18 +78,12 @@ rm $TARBALL.asc
 cd "$BUILDDIR"
 sudo make install
 # restrict to files beginning with n* to leave out shared objects
-tar czvf notcurses-doc-$VERSION.tar.gz n*.1 n*.3 *.html
+tar czvf notcurses-doc-$VERSION.tar.gz n*.1 t*.1 n*.3 *.html
 github-asset dankamongmen/notcurses upload v$VERSION notcurses-doc-$VERSION.tar.gz
 cd ../cffi
 python3 setup.py sdist
 python3 setup.py build
 twine upload -s -udankamongmen dist/*
-cd ../rust
-cargo clean
-cargo publish
-
-cd ../tools/
-./rustdoc-update-gh-pages.sh
 
 cd "../$BUILDDIR"
 cat install_manifest.txt | sudo xargs rm
